from typing import Any
from django.db import models
from django.db.models.query import QuerySet
from django.views.generic import TemplateView, ListView, DetailView
from django.views.generic.detail import SingleObjectMixin
from django.utils.functional import cached_property
from django.urls import reverse
from rest_framework.renderers import TemplateHTMLRenderer
# Create your views here.
from baykeshop.common.mixins import LoginRequiredMixin
from baykeshop.common.renderers import TemplateHTMLRenderer
from baykeshop.apps.shop.api.views import BaykeAddressViewSet
from baykeshop.apps.user.models import BaykeUserBalanceLog
from baykeshop.apps.system.models import (
    BaykeADPosition, BaykeComment, BaykeADSpace
) 
from baykeshop.apps.shop.models import (
    BaykeShopCategory, BaykeShopSPU, BaykeShopSKU, BaykeShopCart,
    BaykeShopOrder
)


    
class SPUAliasAnnotateMixin:
    
    def alias_annotate_queryset(self, queryset):
        return queryset.alias(
            price=models.Min('baykeshopsku__price'),
            sales=models.Sum('baykeshopsku__sales')
        ).annotate(
            price=models.F('price'),
            sales=models.F('sales')
        )


class HomeTemplateView(TemplateView, SPUAliasAnnotateMixin):
    """ 首页 """
    template_name = "shop/index.html"

    def get_context_data(self, **kwargs: Any) -> dict[str, Any]:
        context = super().get_context_data(**kwargs)
        context['title'] = 'baykeShop开源商城系统'
        context['banners'] = self.banners
        context['floors'] = self.get_floors
        return context
    
    @cached_property
    def get_floors(self):
        cates = BaykeShopCategory.objects.filter(is_nav=True, parent__isnull=True)
        for cate in cates:
            subcates = cate.baykeshopcategory_set.filter(is_nav=True)
            cate.spus = self.alias_annotate_queryset(
                BaykeShopSPU.objects.filter(category__in=subcates).distinct()
            )[:10]
        return cates
    
    @cached_property
    def banners(self):
        instance, iscreated = BaykeADPosition.objects.get_or_create(
            name="商城首页轮播图广告位",
            slug="shophomebanner",
            desc="商城首页轮播图广告位【多图】"
        )
        if iscreated:
            BaykeADSpace.objects.create(
                name="轮播图1",
                slug="shopbanner",
                space="img",
                position=instance,
                img="common/ad/banner.jpg"
            )
        return BaykeADPosition.get_position_spaces(slug="shophomebanner")
    

class BaykeShopSPUListView(ListView, SPUAliasAnnotateMixin):
    """ 全部商品 """

    model = BaykeShopSPU
    template_name = "shop/list.html"
    paginate_by = 20
    paginate_orphans = 4
    ordering = "add_date"

    def get_queryset(self) -> QuerySet[Any]:
        queryset = super().get_queryset().filter(status=True)
        return self.alias_annotate_queryset(queryset)
    
    def get_context_data(self, **kwargs: Any) -> dict[str, Any]:
        context = super().get_context_data(**kwargs)
        context['parent_cates'] = self.get_baykeshopcategory
        context['sub_cates'] = self.get_baykeshopcategory_set
        context['title'] = "全部商品"
        context['sort_params'] = self.get_sort_params
        return context
    
    @cached_property
    def get_baykeshopcategory(self):
        parent_cates = BaykeShopCategory.objects.filter(
            status=True, parent__isnull=True
        )
        return parent_cates
    
    @cached_property
    def get_baykeshopcategory_set(self):
        queryset = BaykeShopCategory.objects.none()
        if self.get_baykeshopcategory.exists():
            queryset = self.get_baykeshopcategory.first() \
                .baykeshopcategory_set.filter(status=True)
        return queryset
    
    def alias_annotate_queryset(self, queryset):
        # 对注解过的queryset进行排序
        return super().alias_annotate_queryset(queryset).order_by(self.get_sort_params)
    
    @cached_property
    def get_sort_params(self):
        # 排序字段
        return self.request.GET.get('ordering', self.ordering)
        
    def get_absolute_url(self):
        return reverse('shop:goods')


class BaykeShopCategoryDetailView(SingleObjectMixin, BaykeShopSPUListView):
    """ 商品分类详情列表 """

    def get(self, request, *args, **kwargs):
        self.object = self.get_object(
            queryset=BaykeShopCategory.objects.filter(status=True)
        )
        return super().get(request, *args, **kwargs)

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        context['sub_cates'] = self.get_baykeshopcategory_set
        context['title'] = self.object.name
        return context

    def get_queryset(self) -> QuerySet[Any]:
        queryset = self.object.baykeshopspu_set.filter(status=True)
        if self.object.parent is None:
            queryset = BaykeShopSPU.objects.filter(
                status=True, 
                category__in=self.object.baykeshopcategory_set.filter(status=True)
            )
        return self.alias_annotate_queryset(queryset)
    
    @cached_property
    def get_baykeshopcategory_set(self):
        queryset = self.object.baykeshopcategory_set.filter(status=True)
        if self.object.parent:
            queryset = self.object.parent.baykeshopcategory_set.filter(status=True)
        return queryset
    
    def get_absolute_url(self):
        return reverse('shop:cate-detail', kwargs={'pk': self.pk})


class BaykeShopSPUSearchView(BaykeShopSPUListView):
    """ 搜索视图 """

    def get_queryset(self) -> QuerySet[Any]:
        words = self.request.GET.get('words', '')
        return super().get_queryset().filter(
            models.Q(title__icontains=words)|
            models.Q(subtitle__icontains=words)|
            models.Q(content__icontains=words)
        )

    def get_context_data(self, **kwargs: Any) -> dict[str, Any]:
        context = super().get_context_data(**kwargs)
        context['title'] = self.request.GET.get('words')
        return context


class BaykeShopSPUDetailView(DetailView):
    """ 商品详情页 """
    model = BaykeShopSPU
    context_object_name = "spu"
    template_name = "shop/detail.html"

    def get_context_data(self, **kwargs: Any) -> dict[str, Any]:
        context = super().get_context_data(**kwargs)
        context['title'] = self.get_object().title
        context['spuhots'] = BaykeShopSPU.get_hots()
        context['comments'] = self.get_page_comments()
        return context

    def get_absolute_url(self):
        return reverse('shop:spu-detail', kwargs={'pk': self.pk})
    
    def get_comments(self):
        # 商品评价
        comments = BaykeComment.objects.filter(tag=str(self.get_object().id))
        return comments
    
    def get_page_comments(self):
        # 评价分页
        comments = self.get_comments()
        from django.core.paginator import Paginator
        paginator = Paginator(comments, 20)
        page_number = self.request.GET.get("page")
        page_obj = paginator.get_page(page_number)
        return page_obj
    

class BaykeShopCartListView(LoginRequiredMixin, ListView):
    """ 购物车列表 """
    model = BaykeShopCart
    template_name = "shop/carts.html"
    context_object_name = "carts"

    def get_context_data(self, **kwargs: Any) -> dict[str, Any]:
        context = super().get_context_data(**kwargs)
        context['title'] = '购物车'
        context['carts_values'] = self.get_queryset_values
        return context

    def get_queryset(self) -> QuerySet[Any]:
        return super().get_queryset().filter(owner=self.request.user)
    
    @cached_property
    def get_queryset_values(self):
        cart_queryset = self.get_queryset().values('id', 'sku', 'num', 'add_date')
        for cart in cart_queryset:
            sku = BaykeShopSKU.objects.get(id=cart['sku'])
            cart['sku'] = {
                "id": sku.id,
                "title": sku.spu.title,
                "price": sku.price,
                "img": sku.img.url,
                "cost_price": sku.cost_price,
                "discount_price": sku.discount_price,
                "stock": sku.stock,
                "sales": sku.sales,
                "code": sku.code,
                "specs": { key: value for key, value in sku.specs.items() if not key.isdigit() }
            }
        return list(cart_queryset)
    

class BaykeShopOrderCashDetailView(LoginRequiredMixin, DetailView):
    """ 订单收银台 """
    model = BaykeShopOrder
    template_name = "shop/ordercash.html"
    context_object_name = "order"

    def get_queryset(self) -> QuerySet[Any]:
        return super().get_queryset().filter(owner=self.request.user)

    def get_context_data(self, **kwargs: Any) -> dict[str, Any]:
        context = super().get_context_data(**kwargs)
        context['title'] = "收银台"
        return context
    

class BaykeUserMemberView(LoginRequiredMixin, TemplateView):
    """ 个人中心 """
    template_name = "shop/member/member.html"

    def get_context_data(self, **kwargs: Any) -> dict[str, Any]:
        context = super().get_context_data(**kwargs)
        context['title'] = "个人中心"
        return context
    

class BaykeAddressView(BaykeAddressViewSet):
    """ 用户地址 """
    renderer_classes = [TemplateHTMLRenderer, ]
    
    def list(self, request, *args, **kwargs):
        response = super().list(request, *args, **kwargs)
        response.template_name = "shop/member/address.html"
        return response
    

class BaykeShopOrderListView(LoginRequiredMixin, ListView):
    """ 我的订单 """
    model = BaykeShopOrder
    template_name = "shop/member/orders.html"
    paginate_by = 5

    def get_context_data(self, **kwargs: Any) -> dict[str, Any]:
        context = super().get_context_data(**kwargs)
        context['title'] = '我的订单'
        return context

    def get_queryset(self) -> QuerySet[Any]:
        status = self.request.GET.get('status')
        queryset = super().get_queryset().filter(owner=self.request.user)
        if status:
            queryset = queryset.filter(status=status)
        return queryset


class BaykeShopOrderDetailView(LoginRequiredMixin, DetailView):
    """ 订单详情 """
    model = BaykeShopOrder
    template_name = "shop/member/orders-detail.html"
    context_object_name = "order"

    def get_queryset(self) -> QuerySet[Any]:
        return super().get_queryset().filter(owner=self.request.user)
    
    def get_context_data(self, **kwargs: Any) -> dict[str, Any]:
        context = super().get_context_data(**kwargs)
        context['title'] = "订单详情"
        return context


class BaykeUserBalanceLogTemplateView(LoginRequiredMixin, TemplateView):
    """ 余额变动记录日志 """

    template_name = "shop/member/balance.html"

    def get_context_data(self, **kwargs: Any) -> dict[str, Any]:
        context = super().get_context_data(**kwargs)
        context['add_sum_amount'] = BaykeUserBalanceLog.add_sum_amount(self.request.user)
        context['minus_sum_amount'] = BaykeUserBalanceLog.minus_sum_amount(self.request.user)
        context['balance_list'] = self.get_queryset()
        context['title'] = '我的余额'
        return context
    
    def get_queryset(self):
        return BaykeUserBalanceLog.objects.filter(owner=self.request.user)
    

class BaykeShopOrderCommentView(LoginRequiredMixin, SingleObjectMixin, ListView):
    """ 订单评价 """

    template_name = "shop/member/comment.html"
    paginate_by = 1000

    def get(self, request, *args, **kwargs):    
        self.object = self.get_object(
            queryset=BaykeShopOrder.objects.filter(status=4, owner=request.user)
        )
        return super().get(request, *args, **kwargs)
    
    def get_queryset(self) -> QuerySet[Any]:
        return self.object.baykeshopordersku_set.filter(is_commented=False)
    
    def get_context_data(self, **kwargs: Any) -> dict[str, Any]:
        context = super().get_context_data(**kwargs)
        context['title'] = "订单评价"
        return context


#!/usr/bin/env python
# -*- encoding: utf-8 -*-
'''
@文件    :client.py
@说明    :支付宝支付客户端方法
@时间    :2023/10/31 10:09:37
@作者    :幸福关中&轻编程
@版本    :1.0
@微信    :baywanyun
'''


import logging

from django.conf import settings
from alipay.aop.api.AlipayClientConfig import AlipayClientConfig
from alipay.aop.api.DefaultAlipayClient import DefaultAlipayClient

from baykeshop.common.utils import get_cache_space


logging.basicConfig(
    level=logging.INFO,
    format='%(asctime)s %(levelname)s %(message)s',
    filemode='a',)

logger = logging.getLogger('')

_server_url = 'https://openapi-sandbox.dl.alipaydev.com/gateway.do' if settings.DEBUG else 'https://openapi.alipay.com/gateway.do'


def client(app_id=None, app_private_key=None, alipay_public_key=None):
    """ 构造一个支付宝请求客户端 """
    alipay_client_config = AlipayClientConfig(sandbox_debug=settings.DEBUG)
    alipay_client_config.app_id = app_id or get_cache_space('alipay_APPID')
    alipay_client_config.app_private_key = app_private_key or get_cache_space('alipay_PRIVATE_KEY')
    alipay_public_key = alipay_public_key or get_cache_space('alipay_PUBLIC_KEY')
    alipay_client_config.server_url = _server_url
    return DefaultAlipayClient(alipay_client_config=alipay_client_config, logger=logger)
    


'''
@file            :views.py
@Description     :支付宝支付回调视图
@Date            :2023/09/23 22:28:54
@Author          :幸福关中 && 轻编程
@version         :v1.0
@EMAIL           :1158920674@qq.com
@WX              :baywanyun
'''


from datetime import datetime
from decimal import Decimal

from django.utils import timezone
from django.views.generic import View
from django.shortcuts import render, redirect, HttpResponse
from django.contrib import messages

from baykeshop.common.mixins import LoginRequiredMixin
from baykeshop.apps.shop.models import BaykeShopOrder
from baykeshop.apps.user.models import BaykeUserBalanceLog
from . import mixins


class AliPayCallBackView(View, mixins.AlipayCallBackVerifySignMixin):
    """ PC支付宝支付成功回调 """
    
    def get(self, request, *args, **kwargs):
        # 验签通过处理逻辑
        data = request.GET.dict()
        order_queryset = BaykeShopOrder.objects.filter(order_sn=data['out_trade_no'])
        instance = order_queryset.first()
        if self.has_verify_sign(data):
            paytime = datetime.strptime(data['timestamp'], "%Y-%m-%d %H:%M:%S")
            order_queryset.update(pay_time=timezone.make_aware(paytime), 
                                  total_price=data['total_amount'], 
                                  paymethod=1, status=2)
            return render(request, 'shop/payok.html', {'order': instance})
    
    def post(self, request, *args, **kwargs):
        data = request.POST.dict()
        if self.has_verify_sign(data):
            order_queryset = BaykeShopOrder.objects.filter(order_sn=data['out_trade_no'])
            instance = order_queryset.first()
            if not instance.pay_time:
                paytime = datetime.strptime(data['timestamp'], "%Y-%m-%d %H:%M:%S")
                order_queryset.update(pay_time=timezone.make_aware(paytime), 
                                      total_price=data['total_amount'], paymethod=1, status=2)
            return HttpResponse("success")
        

class BaykeUserBalanceCallBackView(LoginRequiredMixin, AliPayCallBackView):
    """ 余额充值回调 """
    def get(self, request, *args, **kwargs):
        # 验签通过处理逻辑
        data = request.GET.dict()
        baykeuser = request.user.baykeuser
        if self.has_verify_sign(data):
            baykeuser.balance += Decimal(data['total_amount'])
            baykeuser.save()
            BaykeUserBalanceLog.objects.create(
                owner=request.user, 
                amount=Decimal(data['total_amount']),
                change_status=1,
                change_way=1
            )
            messages.success(request, '充值成功！')
        return redirect('shop:member')
    
    def post(self, request, *args, **kwargs):
        # 验签通过处理逻辑
        data = request.POST.dict()
        baykeuser = request.user.baykeuser
        if self.has_verify_sign(data):
            baykeuser.balance += Decimal(data['total_amount'])
            baykeuser.save()
            BaykeUserBalanceLog.objects.create(
                owner=request.user, 
                amount=Decimal(data['total_amount']),
                change_status=1,
                change_way=1
            )
        return HttpResponse('success')


'''
@file            :form.py
@Description     :Admin Form重写
@Date            :2023/09/03 14:19:00
@Author          :幸福关中 && 轻编程
@version         :v1.0
@EMAIL           :1158920674@qq.com
@WX              :baywanyun
'''

from django import forms
from django.contrib.admin.forms import AdminAuthenticationForm
from captcha.fields import CaptchaField

from . import widgets


class BuefyFormMixin(forms.Form):
    """ buefy表单模版 """
    template_name_buefy = "system/buefy.html"
    label_position = 'on-border'  # '' or 'on-border' or 'inside'

    def as_buefy(self):
        """Render as <div> elements."""
        return self.render(self.template_name_buefy)

    def get_context(self):
        context = super().get_context()
        context['labelPosition'] = self.label_position
        
        # 模板中可通过 {{ field.field.widget.attrs }}
        context['attrs_dict'] = {}
        for field in context['form']:
            field_name = field.name
            if 'attrs' in field.field.widget.__dict__:
                context['attrs_dict'][field_name] = field.field.widget.__dict__['attrs']
        return context


class AdminLoginForm(AdminAuthenticationForm, BuefyFormMixin):
    """ 登录后台表单 """
    captcha = CaptchaField(label="验证码")

    def __init__(self, request=None, *args, **kwargs):
        super().__init__(request, *args, **kwargs)
        self.fields['username'].widget = widgets.TextInput(attrs={
            "placeholder": "请输入用户名",
            "icon": "account"
        })
        self.fields['password'].widget = widgets.PasswordInput(attrs={
            "placeholder": "请输入密码",
            "icon": "lock"
        })
        # self.fields['captcha'].label = "验证码"
        self.fields['captcha'].widget = widgets.CaptchaTextInput(attrs={
            "class": "input",
            "placeholder": "请输入验证码",
            "maxlength": 4
        })

    class Media:
        js = ("admin/js/vendor/jquery/jquery.min.js",)


'''
@file            :urls.py
@Description     :接口url
@Date            :2023/09/11 21:08:48
@Author          :幸福关中 && 轻编程
@version         :v1.0
@EMAIL           :1158920674@qq.com
@WX              :baywanyun
'''

from django.urls import path
from rest_framework.routers import DefaultRouter
from . import views

app_name = "shopapi"

router = DefaultRouter()

router.register('address', views.BaykeAddressViewSet, basename='address')

urlpatterns = [
    path(
        'create-cart/', 
        views.BaykeShopCartCreateAPIView.as_view(), 
        name='create-cart'
    ),
    path(
        'update-cart-num/', 
        views.BaykeShopCartUpdateNumAPIView.as_view(), 
        name='update-cart-num'
    ),
    path(
        'del-cart/', 
        views.BaykeShopCartUpdateNumAPIView.as_view(), 
        name='del-cart'
    ),
    path(
        'create-order/', 
        views.BaykeShopOrderCreateAPIView.as_view(), 
        name='create-order'
    ),
    path(
        'cash-order/', 
        views.BaykeShopOrderCashAPIView.as_view(), 
        name='cash-order'
    ),
    path(
        'confirm-receipt/', 
        views.ConfirmReceiptAPIView.as_view(), 
        name='confirm-receipt'
    ),
    *router.urls
]


from django.core import management
from django.core.management.base import BaseCommand, CommandError
from django.conf import settings


class Command(BaseCommand):
    help = "重建数据库"

    def handle(self, *args, **options):
        for app in (settings.BASE_DIR / "baykeshop/apps").iterdir():
            if not app.is_dir() and ("__pycache__" in str(app)):
                continue
            if (app / 'migrations').is_dir():
                for m in (app / 'migrations').iterdir():
                    if m.name == "__pycache__" and m.is_dir():
                        for pycache in m.iterdir():
                            pycache.unlink()
                            self.stdout.write(self.style.SUCCESS('删除 "%s"成功' % str(pycache)))
                        m.rmdir()
                        self.stdout.write(self.style.SUCCESS('删除 "%s"成功' % str(m)))
                    if m.is_file() and m.name != "__init__.py":
                        m.unlink()
                        self.stdout.write(self.style.SUCCESS('删除 "%s"成功' % str(m)))

        management.call_command("makemigrations", verbosity=0, interactive=False)
        management.call_command("migrate", verbosity=0, interactive=False)
        self.stdout.write(self.style.SUCCESS('数据重建成功！'))


from typing import Any
from django.contrib.auth import views
from django.urls import reverse_lazy
from django.contrib.messages.views import SuccessMessageMixin
from django.contrib.auth import authenticate, login
from django.views.generic.edit import FormView
# Create your views here.
from .forms import RegisterForm, LoginForm


class LoginView(SuccessMessageMixin, views.LoginView):
    """ 前台登录 """
    next_page = reverse_lazy("shop:home")
    template_name = "system/login.html"
    form_class = LoginForm
    success_message = "%(username)s 登录成功！"

    def get_success_message(self, cleaned_data):
        return self.success_message % dict(
            cleaned_data,
            username=cleaned_data['username'],
        )

    def get_context_data(self, **kwargs: Any) -> dict[str, Any]:
        context = super().get_context_data(**kwargs)
        context['title'] = "baykeShop"
        context['site_title'] = "登录"
        return context
    

class LogoutView(SuccessMessageMixin, views.LogoutView):
    """ 前台退出 """
    template_name = "system/logout.html"
    next_page = reverse_lazy('shop:home')


class RegisterView(SuccessMessageMixin, FormView):
    """ 注册用户 """
    template_name = 'system/register.html'
    form_class = RegisterForm
    success_url = reverse_lazy("shop:home")
    success_message = "%(username)s 注册成功，已登录！"
    
    def form_valid(self, form):
        if form.is_valid():
            new_user = form.save()
            auth_user = authenticate(username=new_user.username, 
                                     password=form.cleaned_data['password1'])
            login(self.request, auth_user)
        return super().form_valid(form)

    def get_success_message(self, cleaned_data):
        return self.success_message % dict(
            cleaned_data,
            username=cleaned_data['username'],
        )

    def get_context_data(self, **kwargs: Any) -> dict[str, Any]:
        context = super().get_context_data(**kwargs)
        context['title'] = "baykeShop"
        context['site_title'] = "注册"
        return context


from django.contrib import admin
from django.contrib.auth import get_user_model
from django.contrib.auth.admin import UserAdmin
from django.utils.html import format_html
# Register your models here.
from baykeshop.common.options import StackedInline
from .models import BaykeUser

admin.site.unregister(get_user_model())


class BaykeUserInline(StackedInline):
    '''Tabular Inline View for BaykeUser'''

    model = BaykeUser
    min_num = 1
    max_num = 1
    extra = 1
    readonly_fields = ('balance',)

    
@admin.register(get_user_model())
class BaykeUserAdmin(UserAdmin):
    '''Admin View for UserAdmin'''
    list_display = ("username", "avatar", 'sex', 'phone', "balance", "is_staff")
    inlines = [
        BaykeUserInline,
    ]

    @admin.display(description="头像")
    def avatar(self, obj):
        return format_html(
            "<img width='36px' height='36px' src={} />", 
            obj.baykeuser.avatar.url if obj.baykeuser.avatar else '/static/img/avatar.png'
        )
    
    @admin.display(description="余额")
    def balance(self, obj):
        return obj.baykeuser.balance
    
    @admin.display(description="性别")
    def sex(self, obj):
        return obj.baykeuser.get_sex_display()
    
    @admin.display(description="手机号")
    def phone(self, obj):
        return obj.baykeuser.phone
    


'''
@file            :options.py
@Description     :ModelAdmin相关
@Date            :2023/09/04 10:41:08
@Author          :幸福关中 && 轻编程
@version         :v1.0
@EMAIL           :1158920674@qq.com
@WX              :baywanyun
'''

from django.contrib import admin


class ModelAdmin(admin.ModelAdmin):
    pass


class TabularInline(admin.TabularInline):
    pass


class StackedInline(admin.StackedInline):
    pass


'''
@file            :articletags.py
@Description     :内容标签，博客侧边栏
@Date            :2023/09/04 17:07:30
@Author          :幸福关中 && 轻编程
@version         :v1.0
@EMAIL           :1158920674@qq.com
@WX              :baywanyun
'''

from django.template import Library
from baykeshop.apps.article.models import (
    BaykeArticleCategory, BaykeArticleContent,
    BaykeArticleTags
)

register = Library()


@register.inclusion_tag('article/sidebar/category.html', takes_context=True)
def sidebar_category(context):
    """ 文章分类 """
    object = context.get('object')
    category_list = BaykeArticleCategory.objects.exclude(parent__isnull=False)
    return {
        'category_list': category_list,
        'object': object
    }


@register.inclusion_tag('article/sidebar/archiving.html', takes_context=True)
def sidebar_archiving(context):
    """ 文章归档 """
    dates = BaykeArticleContent.objects.dates(field_name="add_date", kind="month")
    return {
        'dates':dates,
        'month': context.get('month')
    }


@register.inclusion_tag('article/sidebar/tags.html')
def sidebar_tags():
    """ 标签 """
    return {
        'tags': BaykeArticleTags.objects.all(),
        'colors': [
            'is-white', 'is-black', 'is-light', 'is-dark', 
            'is-primary', 'is-info', 'is-success', 'is-warning', 'is-danger'
        ],
    }


# Generated by Django 4.2.4 on 2023-11-04 13:33

import baykeshop.common.validators
from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='BaykeUserBalanceLog',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('add_date', models.DateTimeField(auto_now_add=True, verbose_name='添加时间')),
                ('pub_date', models.DateTimeField(auto_now=True, verbose_name='修改时间')),
                ('is_delete', models.BooleanField(default=False, editable=False, verbose_name='删除标记')),
                ('amount', models.DecimalField(decimal_places=2, max_digits=15, verbose_name='金额')),
                ('change_status', models.PositiveSmallIntegerField(blank=True, choices=[(1, '增加'), (2, '支出')], null=True)),
                ('change_way', models.PositiveSmallIntegerField(choices=[(1, '线上充值'), (2, '管理员手动更改'), (3, '余额抵扣商品')], default=2)),
                ('owner', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL, verbose_name='用户')),
            ],
            options={
                'verbose_name': '余额明细',
                'verbose_name_plural': '余额明细',
            },
        ),
        migrations.CreateModel(
            name='BaykeUser',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('add_date', models.DateTimeField(auto_now_add=True, verbose_name='添加时间')),
                ('pub_date', models.DateTimeField(auto_now=True, verbose_name='修改时间')),
                ('is_delete', models.BooleanField(default=False, editable=False, verbose_name='删除标记')),
                ('name', models.CharField(max_length=50, verbose_name='姓名')),
                ('phone', models.CharField(blank=True, default='', max_length=11, validators=[baykeshop.common.validators.validate_phone], verbose_name='手机号')),
                ('sex', models.PositiveSmallIntegerField(choices=[(0, '未知'), (1, '男'), (2, '女')], default=0, verbose_name='性别')),
                ('about', models.CharField(blank=True, default='我喜欢baykeshop这个程序！', max_length=150, verbose_name='简介')),
                ('avatar', models.ImageField(blank=True, max_length=200, null=True, upload_to='avatar/', verbose_name='头像')),
                ('balance', models.DecimalField(blank=True, decimal_places=2, default=0, max_digits=8, verbose_name='余额')),
                ('owner', models.OneToOneField(on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL, verbose_name='用户')),
            ],
            options={
                'verbose_name': '用户',
                'verbose_name_plural': '用户',
                'ordering': ['-add_date'],
            },
        ),
    ]



from django.apps import AppConfig


class UserConfig(AppConfig):
    default_auto_field = 'django.db.models.BigAutoField'
    name = 'baykeshop.apps.user'

    def ready(self) -> None:
        from . import signals



'''
@file            :settings.py
@Description     :默认配置
@Date            :2023/09/02 23:32:44
@Author          :幸福关中 && 轻编程
@version         :v1.0
@EMAIL           :1158920674@qq.com
@WX              :baywanyun
'''


import datetime


DEFAULTS_CONF = {
    # 站点title
    "SITE_TITLE": "baykeShop开源商城系统",
    # 站点头Logo
    "SITE_HEADER": "baykeShop Header",
    # 首页title
    "INDEX_TITLE": "baykeShop开源商城系统",
    # 数据表前缀
    "DB_PREFIX": "bayke",
    # 默认管理后台开启自定义菜单
    "CUSTOM_MENU": False,
    # 手机号验证正则
    "REGEX_PHONE": r"^1[358]\d{9}$|^147\d{8}$|^176\d{8}$",
    # 邮箱验证正则
    "REGEX_EMAIL": r"^\w+([\.-]?\w+)*@\w+([\.-]?\w+)*(\.\w{2,3})+$",
    # 邮箱验证码过期时间
    "EMAIL_CODE_EXP": datetime.timedelta(seconds=300),
    # 验证码随机范围
    "CODE_CHAR": "1234567890",
    # 验证码长度
    "CODE_LENGTH": 4,
    # 支付宝支付配置
    "ALIPAY": {
        "APPID": '2021000122666025',
        "PUBLIC_KEY": 'MIIBIjANBgkqhkiG9w0BAQEFAAOCAQ8AMIIBCgKCAQEAnGm4s+aI1LsCwL9i67GSqEICxqfkyzydk6TIA57M9++EHBUAsXHa5mpgUIQwlYKetzsdPnYK8rS7Pkn6RFogF639zoaRBIpCu8K/lazUB2PqykLdV+XH8DqtH3k6lz1hFRAOHDIn3wVqIUOC0H/G4TFsp8Cd/cgLYAFr12Fgm20/OW8GZNnhfZmmcbHc4el9CqWuEt1xRQLAgLiaDjTZ5RrgSwHem2p1kYKjcs0jw1M+IyKIZK0k6s/KwwqsSlG28ysP94nBI1v3vSIL0e25rvv5irYXi78hfmmO8+sBdYxBzkaF7tndTctTxTYMcnk/+1jKijahDW/72zTI0AwSxwIDAQAB',
        "PRIVATE_KEY": 'MIIEpQIBAAKCAQEAwCuqnkSxuJthPdkQIW2k/l4fpDjWT3TPBQy5hOfKUBUUl42xN72fqtWB8fUAmLlLXuDgFzPMY17Hk8huCDFT75Sidy1nLCayKgo4xnoyk+e/VkUQjEprpRkUq0YzZWPUwQK9aSaO+XI+ivkbnDIBw9fmKV+PxLqfnOPH5630SdT5mEF5hhrwmw8RDrUz404u+BilZw0Q2qJshgXES9vafqpTpe/jiUz8A66v9eIAOskcMDTz4iUuCThH2YCQqWtyE6aNzO3NQ+SVpdBfZNQzmqoRlKDUiX8Fb29sdzBPjSHstYr5rXCVI5XFeCFCqaCpzkpnPoX0DfKIGfmW27UWTQIDAQABAoIBAH97eFBOGefxi+zPlpfWeOIV5r+2xNUpFDyxWFEGw3ukwGhZovrrXISj5vRwJ9ko8wsZ+w7pWemB8rvi9ruQnbIN1EljYVI0yXSCG2/vmKXt00yTvIqUxBxeKwwEIgxc0cLI4oZjAe4RD9R5tQtFUU5uZ02DC4jilQgrQ246RBjFfctF4uZCP9LtToKDGlw1rW6He40CUxhyGVtgNGKx+eK+pspDgdTkcwggVwNWckuWBVBLdVty/J9D9vSNNXFltBHnb27NuRiWLjKHrWDJSxi2DiRDt5Djh/ghmLPlQm3rrqOcUcZHFlfuV3w8SMkjFoY4cSNdbuY9+S8aaPTcGbECgYEA5/I6ToS2/7koi65YN3RB6p8d+8GwF0gttisJu/fRNowQODYATiQWLbRMtUC7zeHq8ee8hPgmz/7rT44GgjPj9Z5GLkTnxaJYXOONCslBRQ+ntIct3oRNCcN/2e2mvfYx5aHnyXek69H6itRFY4czblXcWpZAkoIbNMi9uQtKt0cCgYEA1Bl22DcJsj9aXvBJmLyt2yJUjZdWhUwMXGWfEcGM1teJQfD+d/mcCcFKMhVI7ZVTvNFPZNDuU8H9IovYxps+NBTtzWrhpALHm0cdn1DY7sRIrTbMLn9S89qb7hRxV6FD2auV32I8RAWqrvRLY99A7kFYUEed8436tK4VQyr+t8sCgYEAsk9nP6GLRYBEiU1YiBkZ65KzbC10gBF/AsKHUzeqYHLArVXmWiwn2K75IYZIWnhJM/rg1KAoxlHE+H3IxyO5JcOtVDiorSinIZHVhes+ACeO15vsSVoQF2dxzEmEnBi+NziGhj0yThA4ua2CQodXpIThR8qmjXr0C1ofcDgcElUCgYEAtO1X7M7eTZgvXec50Lm60MBzQilD3AdoT/U8ASiLHMXVx1NtryhVTBj/UsPHZyvHt70RLd3wP7CuX6bN73WEVWc5B87R1lessC+0/C86Lktv95pUCKICQBROiYQUv3zIZUkyWtDwudHfrMil+vb564QActL00Ute/nu1lYt6p4sCgYEArfrdz9R4sgsFxoMsaKd9ya7m/Rz+pPWhWOB27PMr7iSKpo6SL7iwM+hUl8M68h7LzbET7C2CYkUtwiSxZDWUH6u5bmyKGXiY8OEyatyvJtWjFZm/bdUtQi5rtd9/+DcBBKMyMi8x/yWSAxlWjvEbzIVvn38aX6xcLLlVceep5zw='
    },
    # 邮箱配置
    "EMAIL_BACKEND_CONF": {
        "EMAIL_HOST": 'smtp.qq.com',
        "EMAIL_PORT": 465,
        "EMAIL_HOST_USER": '2539909370@qq.com',
        "EMAIL_HOST_PASSWORD": 'fhrygoqlndmxebjfzxc',
        "EMAIL_USE_SSL": True,
        "DEFAULT_FORM_EMAIL": '2539909370@qq.com'
    }
}


'''
@file            :urls.py
@Description     :通用接口url
@Date            :2023/09/14 10:42:37
@Author          :幸福关中 && 轻编程
@version         :v1.0
@EMAIL           :1158920674@qq.com
@WX              :baywanyun
'''

from django.urls import path

from . import views

app_name = "systemapi"

urlpatterns = [
    path(
        'order-comment/', 
         views.BaykeShopOrderCommentAPIView.as_view(), 
         name='order-comment'
    ),
]




from django.core.management.base import BaseCommand

from baykeshop.apps.system.models import BaykeADSpace, BaykeADPosition
from baykeshop.conf import bayke_settings

class Command(BaseCommand):
    help = "初始化数据"
    
    def handle(self, *args, **options):
        instance, iscreated = BaykeADPosition.objects.update_or_create(
            slug="systemConf",
            defaults={
                "name": "系统配置",
                "slug": "systemConf",
                "desc": "系统相关配置"
            }
        )
        BaykeADSpace.objects.update_or_create(
            slug="alipay_APPID",
            space="text",
            defaults={
                "name": "支付宝APPID",
                "slug": "alipay_APPID",
                "space": "text",
                "text": bayke_settings.ALIPAY['APPID'],
                "position": instance
            }
        )
        BaykeADSpace.objects.update_or_create(
            slug="alipay_PUBLIC_KEY",
            space="html",
            defaults={
                "name": "支付宝PUBLIC_KEY",
                "slug": "alipay_PUBLIC_KEY",
                "space": "html",
                "text": "",
                "html": bayke_settings.ALIPAY['PUBLIC_KEY'],
                "position": instance
            }
        )
        BaykeADSpace.objects.update_or_create(
            slug="alipay_PRIVATE_KEY",
            space="html",
            defaults={
                "name": "支付宝PRIVATE_KEY",
                "slug": "alipay_PRIVATE_KEY",
                "space": "html",
                "text":"",
                "html": bayke_settings.ALIPAY['PRIVATE_KEY'],
                "position": instance
            }
        )

        for key, value in bayke_settings.EMAIL_BACKEND_CONF.items():
            BaykeADSpace.objects.update_or_create(
                slug=key,
                space="text",
                defaults={
                    "name": key,
                    "slug": key,
                    "text": value,
                    "position": instance
                }
            )
        self.stdout.write(self.style.SUCCESS(f"初始化必要数据成功！"))


from django.contrib import admin
from django.http.request import HttpRequest
from django.urls.resolvers import URLPattern
from django.template.response import TemplateResponse
from django.utils.html import format_html
from django.shortcuts import get_object_or_404, redirect
from django.contrib import messages
from django.contrib.admin.views.decorators import staff_member_required
from django.contrib.auth.decorators import permission_required
from django.utils.decorators import method_decorator
# Register your models here.
from baykeshop.common.options import ModelAdmin
from baykeshop.conf import bayke_settings
from .models import (
    BaykeADPosition, BaykeADSpace, BaykeComment, BaykeSiteMenus
)


@admin.register(BaykeADPosition)
class BaykeADPositionAdmin(ModelAdmin):
    '''Admin View for BaykeADPosition'''

    list_display = ('id', 'slug', 'name', 'add_date')
    prepopulated_fields = {'slug': ('name',),}


@admin.register(BaykeADSpace)
class BaykeADSpaceAdmin(ModelAdmin):
    '''Admin View for BaykeADSpace'''

    list_display = ('id', 'slug', 'name', 'position', 'remark', 'add_date')
    # prepopulated_fields = {'slug': ('name',),}
    list_display_links = ('id', 'slug', )
    list_filter = ('position',)

    def get_readonly_fields(self, request: HttpRequest, obj=None):
        if obj and obj.space == 'text':
            self.readonly_fields = ('html', 'img', 'slug')
        elif obj and obj.space == 'html':
            self.readonly_fields = ('text', 'img', 'target', 'slug')
        elif obj and obj.space == 'img':
            self.readonly_fields = ('text', 'html', 'slug')
        return super().get_readonly_fields(request, obj)


@admin.register(BaykeComment)
class BaykeCommentAdmin(admin.ModelAdmin):
    '''Admin View for BaykeComment'''

    list_display = ('id', 'owner', 'content', 'score', 'content_object', 'reply', 'reply_comment')
    readonly_fields = ('content_type', 'object_id', 'tag', 'owner', 'content', 'score', 'content_object', 'reply')
    list_display_links = ('owner', 'content')

    @admin.display(description="操作")
    def reply_comment(self, obj):
        from django.urls import reverse
        url = reverse('admin:reply_view', args=[obj.id])
        return '已回复' if obj.reply else format_html("<a href='{}'>回复</a>", url)
    
    def get_urls(self) -> list[URLPattern]:
        from django.urls import path
        urls = super().get_urls()
        my_urls = [
            path(
                'reply_view/<int:comment_id>/', 
                 self.admin_site.admin_view(self.reply_view), 
                 name='reply_view'
            ),
        ]
        return my_urls + urls
    
    @method_decorator(staff_member_required)
    @method_decorator(permission_required("system.reply_to_comments", raise_exception=True))
    def reply_view(self, request, comment_id=None):
        obj = get_object_or_404(BaykeComment, id=comment_id)
        context = dict(
            self.admin_site.each_context(request),
            title=f"评论回复",
            obj=obj
        )
        if request.method == 'POST':
            reply = request.POST.get('reply', '')
            obj.reply = reply
            obj.save()
            messages.success(request, '回复成功！')
            return redirect("admin:system_baykecomment_changelist")
        return TemplateResponse(request, 'system/reply_comment.html', context)
    
    def has_add_permission(self, request: HttpRequest) -> bool:
        return False
    
    def has_change_permission(self, request: HttpRequest, obj=None) -> bool:
        return False
    

if bayke_settings.CUSTOM_MENU:
    @admin.register(BaykeSiteMenus)
    class BaykeSiteMenusAdmin(ModelAdmin):
        '''Admin View for BaykeSiteMenus'''

        list_display = ('id', 'name', 'parent', 'permission', 'add_date')


# Generated by Django 4.2.4 on 2023-11-04 13:33

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('contenttypes', '0002_remove_content_type_name'),
        ('auth', '0012_alter_user_first_name_max_length'),
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='BaykeADPosition',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('add_date', models.DateTimeField(auto_now_add=True, verbose_name='添加时间')),
                ('pub_date', models.DateTimeField(auto_now=True, verbose_name='修改时间')),
                ('is_delete', models.BooleanField(default=False, editable=False, verbose_name='删除标记')),
                ('name', models.CharField(max_length=50, verbose_name='名称')),
                ('slug', models.SlugField(unique=True, verbose_name='slug')),
                ('desc', models.CharField(blank=True, default='', max_length=150, verbose_name='说明')),
            ],
            options={
                'verbose_name': '广告位',
                'verbose_name_plural': '广告位',
                'ordering': ['-add_date'],
            },
        ),
        migrations.CreateModel(
            name='BaykeSiteMenus',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('add_date', models.DateTimeField(auto_now_add=True, verbose_name='添加时间')),
                ('pub_date', models.DateTimeField(auto_now=True, verbose_name='修改时间')),
                ('is_delete', models.BooleanField(default=False, editable=False, verbose_name='删除标记')),
                ('name', models.CharField(max_length=50, unique=True, verbose_name='名称')),
                ('icon', models.CharField(blank=True, default='', max_length=50, verbose_name='图标')),
                ('parent', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='system.baykesitemenus')),
                ('permission', models.OneToOneField(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='auth.permission', verbose_name='权限')),
            ],
            options={
                'verbose_name': 'SiteMenus',
                'verbose_name_plural': 'SiteMenus',
                'ordering': ['-add_date'],
            },
        ),
        migrations.CreateModel(
            name='BaykeComment',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('add_date', models.DateTimeField(auto_now_add=True, verbose_name='添加时间')),
                ('pub_date', models.DateTimeField(auto_now=True, verbose_name='修改时间')),
                ('is_delete', models.BooleanField(default=False, editable=False, verbose_name='删除标记')),
                ('object_id', models.PositiveIntegerField(blank=True, null=True)),
                ('tag', models.CharField(blank=True, default='', max_length=200, verbose_name='标记')),
                ('content', models.CharField(max_length=200, verbose_name='评论')),
                ('score', models.PositiveSmallIntegerField(choices=[(1, '差评'), (3, '中评'), (5, '好评')], default=5, verbose_name='评价')),
                ('reply', models.CharField(blank=True, default='', max_length=200, verbose_name='回复内容')),
                ('content_type', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='contenttypes.contenttype')),
                ('owner', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL, verbose_name='用户')),
            ],
            options={
                'verbose_name': '评论',
                'verbose_name_plural': '评论',
                'ordering': ['-add_date'],
                'permissions': [('reply_to_comments', 'Reply to comments')],
            },
        ),
        migrations.CreateModel(
            name='BaykeADSpace',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('add_date', models.DateTimeField(auto_now_add=True, verbose_name='添加时间')),
                ('pub_date', models.DateTimeField(auto_now=True, verbose_name='修改时间')),
                ('is_delete', models.BooleanField(default=False, editable=False, verbose_name='删除标记')),
                ('name', models.CharField(max_length=50, verbose_name='名称')),
                ('slug', models.SlugField(unique=True, verbose_name='slug')),
                ('space', models.CharField(choices=[('text', '文字广告'), ('img', '图片广告'), ('html', '自定义广告')], default='img', max_length=10, verbose_name='类型')),
                ('remark', models.CharField(blank=True, default='', max_length=150, verbose_name='备注')),
                ('img', models.ImageField(blank=True, null=True, upload_to='common/ad', verbose_name='图片广告')),
                ('text', models.CharField(blank=True, default='', max_length=150, verbose_name='文字广告')),
                ('html', models.TextField(blank=True, default='', verbose_name='自定义广告')),
                ('target', models.URLField(blank=True, default='', verbose_name='跳转地址')),
                ('sort', models.PositiveSmallIntegerField(default=1, verbose_name='排序')),
                ('status', models.BooleanField(default=True, verbose_name='状态')),
                ('position', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='system.baykeadposition', verbose_name='广告位')),
            ],
            options={
                'verbose_name': '广告内容',
                'verbose_name_plural': '广告内容',
                'ordering': ['sort'],
            },
        ),
    ]



'''
@file            :serializers.py
@Description     :通用接口序列化
@Date            :2023/09/14 10:29:32
@Author          :幸福关中 && 轻编程
@version         :v1.0
@EMAIL           :1158920674@qq.com
@WX              :baywanyun
'''
from django.contrib.contenttypes.models import ContentType
from rest_framework import serializers

from baykeshop.apps.system.models import BaykeComment
from baykeshop.apps.shop.models import BaykeShopOrderSKU


class BaykeShopOrderCommentSerializer(serializers.Serializer):
    """ 序列化 """
    app_label = serializers.CharField(max_length=80)
    model = serializers.CharField(max_length=80)
    object_id = serializers.IntegerField(min_value=1)
    content = serializers.CharField(max_length=200)
    score = serializers.IntegerField(min_value=1, max_value=5, default=5)
    tag = serializers.CharField(max_length=200, required=False, default="")

    def create(self, validated_data):
        instance = BaykeComment.objects.create(
            owner=self.context['request'].user,
            content_type=validated_data['content_type'],
            object_id=validated_data['object_id'],
            content=validated_data['content'],
            score=validated_data['score'],
            tag=validated_data['tag']
        )
        # 如果是对订单商品评价，则需要执行以下操作
        if isinstance(instance.content_object, BaykeShopOrderSKU):
            ordersku = instance.content_object
            order = ordersku.order
            ordersku.is_commented = True
            ordersku.save()
            is_commented_list = order.baykeshopordersku_set.values_list('is_commented', flat=True)
            if is_commented_list and all(is_commented_list):
                order.status = 5
                order.save()
        return instance
    
    def validate(self, attrs):
        try:
            content_type = ContentType.objects.get(
                app_label=attrs['app_label'],
                model = attrs['model']
            )
            attrs['content_type'] = content_type
        except ContentType.DoesNotExist:
            raise serializers.ValidationError('模型不存在！')
        return attrs


from django.template import Library
from baykeshop.apps.shop.models import BaykeShopCategory
from baykeshop.common.utils import get_cache_space

register = Library()

@register.inclusion_tag("system/pagination.html")
def pagination(page_obj):
    return {
        "count": page_obj.paginator.count,
        "current": page_obj.number,
        "per_page":page_obj.paginator.per_page
    }

@register.simple_tag
def shopcates():
    return BaykeShopCategory.objects.filter(parent__isnull=True, is_nav=True)

@register.filter
def multiply(value, arg):
    # 相乘计算
    from decimal import Decimal
    return Decimal(value) * int(arg)

@register.simple_tag
def space(slug):
    return get_cache_space(slug)




#!/usr/bin/env python
# -*- encoding: utf-8 -*-
'''
@文件    :mixins.py
@说明    :支付宝验签
@时间    :2023/10/31 10:07:40
@作者    :幸福关中&轻编程
@版本    :1.0
@微信    :baywanyun
'''


from alipay.aop.api.util.SignatureUtils import verify_with_rsa
from baykeshop.common.utils import get_cache_space


class AlipayCallBackVerifySignMixin:
    """ 支付宝支付回调，验签 """
    
    def has_verify_sign(self, data):
        """ 验签
        data是从请求中获得的字典数据，携带 sign和sign_type
        """
        sign = data.pop('sign')
        sign_type = data.pop('sign_type')
        alipay_public_key = get_cache_space('alipay_PUBLIC_KEY')
        # 去除sign和sign_type参数之后进行升序排列，拼装请求参数用支付宝公钥进行验签
        message='&'.join([f"{k}={v}" for k, v in dict(sorted(data.items(), key=lambda d: d[0], reverse=False)).items()])
        flag = verify_with_rsa(alipay_public_key, message.encode('UTF-8','strict'), sign)
        return flag


from django.core import management
from django.core.management.base import BaseCommand, CommandError

from django.conf import settings


class Command(BaseCommand):
    help = "导出初始数据"
    
    def handle(self, *args, **options):
        baykeadposition = settings.BASE_DIR / 'baykeshop/conf/baykeadposition.json'
        baykeadspace = settings.BASE_DIR / 'baykeshop/conf/baykeadspace.json'
        baykeshopcategory = settings.BASE_DIR / 'baykeshop/conf/baykeshopcategory.json'
        management.call_command('dumpdata', 'system.baykeadposition', output=baykeadposition, indent=2, format='json')
        management.call_command('dumpdata', 'system.baykeadspace', output=baykeadspace, indent=2, format='json')
        management.call_command('dumpdata', 'shop.baykeshopcategory', output=baykeshopcategory, indent=2, format='json')
        self.stdout.write(self.style.SUCCESS(f"数据导出成功，导出数据路径在{settings.BASE_DIR / 'baykeshop/conf'}文件夹下"))


from django.test import TestCase
from django.shortcuts import get_list_or_404

# Create your tests here.
from .models import BaykeArticleCategory, BaykeArticleContent


class BaykeArticleCategoryTestCase(TestCase):

    def setUp(self):
        # 在每个测试方法之前运行
        self.category1 = BaykeArticleCategory.objects.create(
            name="Category 1",
            desc="This is the first category",
            keywords="Key words for category 1",
            parent=None,
            status=True,
            sort=1,
        )
        self.category2 = BaykeArticleCategory.objects.create(
            name="Category 2",
            desc="This is the second category",
            keywords="Key words for category 2",
            parent=self.category1,
            status=True,
            sort=2,
        )

    def test_category_creation(self):
        """测试分类的创建"""
        category = BaykeArticleCategory.objects.get(name="Category 1")
        self.assertEqual(category.name, "Category 1")
        self.assertEqual(category.desc, "This is the first category")
        self.assertEqual(category.keywords, "Key words for category 1")
        self.assertEqual(category.parent, None)
        self.assertEqual(category.status, True)
        self.assertEqual(category.sort, 1)

    def test_category_parent(self):
        """测试分类的父类"""
        category = BaykeArticleCategory.objects.get(name="Category 2")
        self.assertEqual(category.parent.name, "Category 1")

    def test_category_str(self):
        """测试分类的字符串表示"""
        category = BaykeArticleCategory.objects.get(name="Category 1")
        self.assertEqual(str(category), category.name)


class BaykeArticleContentTest(TestCase):

    def setUp(self):
        # 在每个测试方法之前运行
        self.article_category = BaykeArticleCategory.objects.create(
            name = '测试分类'
        )
        self.draft_content = BaykeArticleContent.objects.create(
            title='草稿标题',
            desc='草稿描述',
            keywords='草稿关键词',
            category=self.article_category, 
            content='草稿内容',
            status=0  # 设置为草稿状态
        )

        self.publish_content = BaykeArticleContent.objects.create(
            title='发布标题',
            desc='发布描述',
            keywords='发布关键词',
            category=self.article_category,
            content='发布内容',
            status=1  # 设置为发布状态
        )

    def test_model_instance(self):
        """测试是否正确创建了 BaykeArticleContent 实例"""
        self.assertTrue(self.draft_content.title == '草稿标题')
        self.assertTrue(self.publish_content.title == '发布标题')

    def test_model_str(self):
        """测试 str() 方法是否返回正确的字符串表示形式"""
        self.assertTrue(str(self.draft_content) == '草稿标题')
        self.assertTrue(str(self.publish_content) == '发布标题')

    def test_model_get_status_display(self):
        """测试 get_status_display() 方法返回的状态显示文本是否正确"""
        self.assertTrue(self.draft_content.get_status_display() == '草稿')
        self.assertTrue(self.publish_content.get_status_display() == '发布')

    def test_model_save_and_reload(self):
        """测试 save() 方法是否正确保存数据，并使用 get() 方法重新加载数据"""
        self.draft_content.title = '更新后的草稿标题'
        self.draft_content.save()
        updated_content = BaykeArticleContent.objects.get(id=self.draft_content.id)
        self.assertTrue(updated_content.title == '更新后的草稿标题')


from django.db import models
from django.contrib.auth import get_user_model
from django.utils.translation import gettext_lazy as _
from django.db.models import Sum
# Create your models here.
from baykeshop.common.models import BaseModelMixin
from baykeshop.common.validators import validate_phone


class BaykeUser(BaseModelMixin):
    """Model definition for BaykeUser."""

    owner = models.OneToOneField(
        get_user_model(), 
        on_delete=models.CASCADE, 
        verbose_name=_("用户")
    )
    name = models.CharField(_("姓名"), max_length=50)
    phone = models.CharField(
        "手机号", 
        blank=True, 
        default="",
        max_length=11, 
        validators=[validate_phone]
    )
    sex = models.PositiveSmallIntegerField(
        choices=((0, "未知"), (1, "男"), (2, "女")), 
        default=0, 
        verbose_name=_("性别")
    )
    about = models.CharField(
        _("简介"), 
        max_length=150,
        default="我喜欢baykeshop这个程序！", 
        blank=True
    )
    avatar = models.ImageField(
        _("头像"), 
        upload_to="avatar/", 
        max_length=200, 
        blank=True, 
        null=True
    )
    balance = models.DecimalField(
        _("余额"), 
        max_digits=8, 
        decimal_places=2, 
        blank=True, 
        default=0
    )

    # TODO: Define fields here

    class Meta:
        """Meta definition for BaykeUser."""
        ordering = ["-add_date"]
        verbose_name = _("用户")
        verbose_name_plural = verbose_name

    def __str__(self):
        """Unicode representation of BaykeUser."""
        return self.name


class BaykeUserBalanceLog(BaseModelMixin):
    """ 用户余额变动表 """
    
    class BalanceChangeStatus(models.IntegerChoices):
        # 收支状态
        ADD = 1, _('增加')
        MINUS = 2, _('支出')
    
    class BalanceChangeWay(models.IntegerChoices):
        # 收支渠道或方式
        PAY = 1, _('线上充值')        
        ADMIN = 2, _('管理员手动更改') 
        SHOP = 3, _('余额抵扣商品')
    
    owner = models.ForeignKey(get_user_model(), on_delete=models.CASCADE, verbose_name="用户")
    amount = models.DecimalField("金额", max_digits=15, decimal_places=2)
    change_status = models.PositiveSmallIntegerField(
        choices=BalanceChangeStatus.choices, 
        blank=True,
        null=True
    )
    change_way = models.PositiveSmallIntegerField(
        choices=BalanceChangeWay.choices, 
        default=BalanceChangeWay.ADMIN        # 默认为后台
    )

    class Meta:
        verbose_name = '余额明细'
        verbose_name_plural = verbose_name

    def __str__(self):
        return f"{self.owner.username}-{self.amount}"
    
    @classmethod
    def balance_queryset(cls, user):
        return cls.objects.filter(owner=user)
    
    @classmethod
    def add_sum_amount(cls, user):
        # 累计充值
        amount__sum = cls.balance_queryset(user).filter(
            change_status=1).aggregate(Sum('amount')).get('amount__sum')
        return round(amount__sum, 2) if amount__sum else 0
    
    @classmethod
    def minus_sum_amount(cls, user):
        # 累计消费
        amount__sum = cls.balance_queryset(user).filter(
            change_status=2).aggregate(Sum('amount')).get('amount__sum')
        return round(amount__sum, 2) if amount__sum else 0


from django.urls import path
from . import views

app_name = "baykearticle"

urlpatterns = [
    path('', views.BaykeArticleContentListView.as_view(), name='article-list'),
    path('category/<int:pk>/', views.BaykeArticleCategoryDetailView.as_view(), name='category-detail'),
    path('content/<int:pk>/', views.BaykeArticleContentDetailView.as_view(), name='content-detail'),
    path('archive/<int:year>/<int:month>/', views.BaykeArticleContentMonthArchiveView.as_view(), name='archive-list'),
    path('tags/<int:pk>/', views.BaykeArticleTagsToArticleListView.as_view(), name='tags-list'),
]


'''
@file            :utils.py
@Description     :小工具
@Date            :2023/09/14 12:31:42
@Author          :幸福关中 && 轻编程
@version         :v1.0
@EMAIL           :1158920674@qq.com
@WX              :baywanyun
'''

import random
import string

from django.core.cache import cache
from django.conf import settings
from django.core.mail import send_mail, get_connection
from django.db.utils import OperationalError

from baykeshop.conf import bayke_settings
from baykeshop.apps.system.models import BaykeADSpace, BaykeADPosition


def code_random(code_length=bayke_settings.CODE_LENGTH):
    """ 生成指定位数随机字符串方法 """
    # chars = string.ascii_letters + string.digits   # 生成a-zA-Z0-9字符串
    chars = string.digits
    strcode = ''.join(random.sample(chars, code_length))  # 生成随机指定位数字符串
    return strcode


def get_email_connection():
    # 邮件后端
    DEVELOP_EMAIL_BACKEND = "django.core.mail.backends.console.EmailBackend"
    PRODUCTION_EMAIL_BACKEND = "django.core.mail.backends.smtp.EmailBackend"
    connection = get_connection(
        # backend="django.core.mail.backends.smtp.EmailBackend", 
        backend=DEVELOP_EMAIL_BACKEND if settings.DEBUG else PRODUCTION_EMAIL_BACKEND,
        fail_silently=False,
        host=get_cache_space('EMAIL_HOST'),
        port=int(get_cache_space('EMAIL_PORT')),
        username=get_cache_space('EMAIL_HOST_USER'),
        password=get_cache_space('EMAIL_HOST_PASSWORD'),
        use_ssl=bool(get_cache_space('EMAIL_USE_SSL'))
    )
    return connection


def push_main(code, email):
    # 发送邮件
    connection = get_email_connection()
    send_mail(
        subject="BaykeShop验证码, 请查收！", 
        message=f"您的验证码为：{code}, 请尽快验证，5分钟内有效！",
        from_email=get_cache_space('DEFAULT_FORM_EMAIL'),
        recipient_list=[email],
        connection=connection
    )


def generate_order_sn(user):
    # 当前时间 + userid + 随机数
    from random import Random
    from django.utils import timezone
    random_ins = Random()
    order_sn = "{time_str}{user_id}{ranstr}".format(
        time_str=timezone.now().strftime("%Y%m%d%H%M%S"),
        user_id=user.id,
        ranstr=random_ins.randint(10, 99))
    return order_sn


def get_cache_space(slug):
    # 缓存配置
    try:
        space = None
        space_obj = BaykeADSpace.get_space(slug)
        if space_obj and space_obj.space == 'text':
            space = space_obj.text
        elif space_obj and space_obj.space == 'html':
            space = space_obj.html
        elif space_obj and space_obj.space == 'img':
            space = space_obj.img.url if space_obj.img else None
        return cache.get_or_set(slug, space)
    except OperationalError:
        pass


def get_cache_position_spaces(slug):
    # 缓存广告位置及所属广告内容
    spaces_dict = {}
    spaces = BaykeADPosition.get_position_spaces(slug)
    spaces_dict[slug] = {}
    if spaces:
        for space in spaces:
            spaces_dict[slug][space.slug] = {
                "name": space.name,
                "slug": space.slug,
                "space": space.space,
                "position": space.position.slug,
                "remark": space.remark,
                "img": space.img.url if space.img else None,
                "text": space.text,
                "html": space.html,
                "target": space.target,
                "status": space.status
            }
    return cache.get_or_set(slug, spaces_dict)


from django.urls import NoReverseMatch, reverse
from baykeshop.apps.system.models import BaykeSiteMenus


class DynamicMenu:
    
    def __init__(self, request) -> None:
        self._request = request
        self._user = request.user

    def is_authenticated(self):
        # 判断当前用户是否登录
        return self._user.is_authenticated
    
    def get_user_perms(self):
        # 获取当前用户的权限标识
        return (
            self._user.get_all_permissions() 
            if self.is_authenticated
            else set()
        )
        
    def _get_queryset(self):
        # 菜单的queryset数据
        return BaykeSiteMenus.objects.body()
    
    def get_permmenus(self):
        # 当前用户拥有权限的菜单
        menus = self._get_queryset()
        perms = self.get_user_perms()
        
        permmenus = []
        for menu in menus:
            if menu.permission:
                app_label = menu.permission.content_type.app_label
                codename = menu.permission.codename
                info = f"{app_label}.{codename}"
                if info in perms:
                    if menu.parent:
                        permmenus.append(menu.parent)
                    permmenus.append(menu)
        return set(permmenus)
    
    def get_queryset(self):
        return self._get_queryset().filter(id__in=[menu.id for menu in self.get_permmenus()])
    

class MenusMixins:
    """ 后端自定义菜单类 """
    def _build_menus(self, request):
        perm_menus = []
        menus_obj = DynamicMenu(request) 
        menus = menus_obj.get_queryset()
        
        for menu in menus:
            model_dict = {
                "id": menu.id,
                "name": menu.name,
                "app_label": menu.name,
                "icon": menu.icon,
                "parent": menu.parent.id if menu.parent else None,
                "expanded": False,
            }
            if menu.permission:
                app_label = menu.permission.content_type.app_label
                model = menu.permission.content_type.model_class()
                
                if model in self._registry.keys():
                    # 获取模型对应的ModelAdmin
                    model_admin = self._registry.get(menu.permission.content_type.model_class())
                    # 判断是否拥有该应用app的权限
                    has_module_perms = model_admin.has_module_permission(request)
                    if not has_module_perms:
                        continue
                    # 获取该模型的默认增删改查权限
                    perms = model_admin.get_model_perms(request)
                    if True not in perms.values():
                        continue
                    
                    info = (app_label, model._meta.model_name)
                    model_dict = {
                        "id": menu.id,
                        "model": model,
                        "name": menu.name,
                        "object_name": model._meta.object_name,
                        "perms": perms,
                        "icon": menu.icon,
                        "admin_url": None,
                        "add_url": None,
                        "parent": menu.parent.id if menu.parent else None,
                        "active": False
                    }
                        
                    if perms.get("change") or perms.get("view"):
                        model_dict["view_only"] = not perms.get("change")
                        try:
            
                            model_dict["admin_url"] = reverse(
                                "admin:%s_%s_changelist" % info, current_app=self.name
                            )
                            if request.path == model_dict.get("admin_url"):
                                model_dict['active'] = True
                                request.breadcrumbs = model_dict
                        except NoReverseMatch:
                            pass
                        
                    if perms.get("add"):
                        try:
                            model_dict["add_url"] = reverse(
                                "admin:%s_%s_add" % info, current_app=self.name
                            )
                        except NoReverseMatch:
                            pass
                else:
                    print(f'{model._meta.model_name}【{model._meta.verbose_name}】未在admin中注册，请先注册模型')
                if not model_dict["admin_url"]:
                    continue
            perm_menus.append(model_dict)
        perm_menus = self.generate_tree(perm_menus, None)
        return perm_menus
    
    def get_menus(self, request):
        return self._build_menus(request)
    
    def generate_tree(self, source, parent):
        """ 树形结构迭代"""
        tree = []
        for item in source:
            if item["parent"] == parent:
                item["models"] = self.generate_tree(source, item["id"])
                tree.append(item)
            self._traverse(item)
        return tree
                
    def _traverse(self, node):
        # 递归判断active并修改父类的expanded
        if 'models' in node and node['models']:
            for item in node['models']:
                if not item.get('active'):
                    continue
                node['expanded'] = True
                self._traverse(item)


from django.core import management
from django.core.management.base import BaseCommand, CommandError
from django.conf import settings


class Command(BaseCommand):
    help = "导入初始数据"
    
    def handle(self, *args, **options):
        baykeadposition = settings.BASE_DIR / 'baykeshop/conf/baykeadposition.json'
        baykeadspace = settings.BASE_DIR / 'baykeshop/conf/baykeadspace.json'
        baykeshopcategory = settings.BASE_DIR / 'baykeshop/conf/baykeshopcategory.json'
        management.call_command('loaddata', baykeadposition, verbosity=0)
        management.call_command('loaddata', baykeadspace, verbosity=0)
        management.call_command('loaddata', baykeshopcategory, verbosity=0)
        self.stdout.write(self.style.SUCCESS("数据导入成功！"))


from django.urls import path, include
from baykeshop.pay.alipay.views import AliPayCallBackView, BaykeUserBalanceCallBackView
from . import views


app_name = "shop"

urlpatterns = [
    path('', views.HomeTemplateView.as_view(), name='home'),
    path('goods/', views.BaykeShopSPUListView.as_view(), name='goods'),
    path('search/', views.BaykeShopSPUSearchView.as_view(), name='search'),
    path('cate/<int:pk>/', views.BaykeShopCategoryDetailView.as_view(), name='cate-detail'),
    path('spu/<int:pk>/', views.BaykeShopSPUDetailView.as_view(), name='spu-detail'),
    path('carts/', views.BaykeShopCartListView.as_view(), name='carts'),
    path('order-cash/<int:pk>/', views.BaykeShopOrderCashDetailView.as_view(), name='order-cash'),
    path('member/', views.BaykeUserMemberView.as_view(), name='member'),
    path('address/', views.BaykeAddressView.as_view({'get': 'list'}), name='address'),
    path('orders/', views.BaykeShopOrderListView.as_view(), name='orders-list'),
    path('orders-detail/<int:pk>/', views.BaykeShopOrderDetailView.as_view(), name='orders-detail'),
    path('balance-log/', views.BaykeUserBalanceLogTemplateView.as_view(), name='balance-log'),
    path('alipay/', AliPayCallBackView.as_view(), name='alipay'),
    path('balance/', BaykeUserBalanceCallBackView.as_view(), name='balance'),
    path('orders/comment/<int:pk>/', views.BaykeShopOrderCommentView.as_view(), name='comment'),

    path('api/', include('baykeshop.apps.shop.api.urls')),
]


from typing import Any
from django.contrib.auth.base_user import AbstractBaseUser
from django.contrib.auth.forms import UserCreationForm

from baykeshop.common.forms import BuefyFormMixin, AdminLoginForm
from baykeshop.common import widgets


class LoginForm(AdminLoginForm):
    """ 登录表单 """
    
    def confirm_login_allowed(self, user: AbstractBaseUser) -> None:
        # 取消验证is_active
        pass


class RegisterForm(UserCreationForm, BuefyFormMixin):
    # 注册表单
    def __init__(self, *args: Any, **kwargs: Any) -> None:
        super().__init__(*args, **kwargs)
        self.fields['username'].widget = widgets.TextInput(attrs={
            "placeholder": "请输入用户名",
            "icon": "account"
        })
        self.fields['password1'].widget = widgets.PasswordInput(attrs={
            "placeholder": "请输入密码",
            "icon": "lock"
        })
        self.fields['password2'].widget = widgets.PasswordInput(attrs={
            "placeholder": "请再次输入密码",
            "icon": "lock"
        })


from django.conf import settings

settings.DEBUG = False

settings.INSTALLED_APPS += [
    'baykeshop.common.AdminConfig',
    'baykeshop.apps.user',
    'baykeshop.apps.article',
    'baykeshop.apps.system',
    'baykeshop.apps.shop',
    'rest_framework',
    'captcha',
    'tinymce',
]

STATIC_ROOT = settings.BASE_DIR / "static"

MEDIA_URL = 'media/'
MEDIA_ROOT = settings.BASE_DIR / 'media'

TINYMCE_CONFIG = {
    'menubar': True,
    'relative_urls': False,
    'toolbar_sticky': False,
    'plugins_exclude': 'upgrade'  # 禁用升级插件
}

SECURE_CROSS_ORIGIN_OPENER_POLICY = None


#!/usr/bin/env python
# -*- encoding: utf-8 -*-
'''
@文件    :trade_page_pay.py
@说明    :支付宝网页支付
@时间    :2023/10/31 10:08:59
@作者    :幸福关中&轻编程
@版本    :1.0
@微信    :baywanyun
'''


from alipay.aop.api.domain.AlipayTradePagePayModel import AlipayTradePagePayModel
# from alipay.aop.api.domain.SettleDetailInfo import SettleDetailInfo
# from alipay.aop.api.domain.SettleInfo import SettleInfo
# from alipay.aop.api.domain.SubMerchant import SubMerchant
from alipay.aop.api.request.AlipayTradePagePayRequest import AlipayTradePagePayRequest
from baykeshop.pay.alipay.client import client


def trade_page_pay(
    out_trade_no='', total_amount=0, subject='', body='', 
    return_url=None, notify_url=None, client=client()
    ):
    """ 网页支付 """
    
    model = AlipayTradePagePayModel()
    model.out_trade_no = out_trade_no
    model.total_amount = total_amount
    model.subject = subject
    model.body = body
    model.product_code = "FAST_INSTANT_TRADE_PAY"
    
    # settle_detail_info = SettleDetailInfo()
    request = AlipayTradePagePayRequest(biz_model=model)
    request.return_url = return_url
    request.notify_url = notify_url
    return client.page_execute(request, http_method="GET")



from django.template import Library
from django.db.models import Sum, Avg

from baykeshop.apps.system.models import BaykeComment
from baykeshop.apps.shop.models import BaykeShopCart

register = Library()


def spudata(spu):
    skus = spu.baykeshopsku_set.all()
    return {
        "img": skus.first().img.url,
        "spu": spu,
    }


@register.inclusion_tag("shop/spubox.html")
def spubox(spu):
    return spudata(spu)


@register.inclusion_tag("shop/specs.html")
def spuspecs(spu):
    skus = spu.baykeshopsku_set.all()
    return {
        "skus": list(skus.values())
    }


@register.inclusion_tag("shop/spubanners.html")
def spubanners(spu):
    return {
        "images": list(spu.baykeshopspuatlas_set.filter(status=True).values("id", "img"))
    }


@register.simple_tag
def cartscount(request):
    # 购物车商品数量
    return (
        BaykeShopCart.get_cart_count(request.user) 
        if request.user.is_authenticated else 0
    )


def ordersku_func(ordersku_queryset):
    from decimal import Decimal
    count__sum = ordersku_queryset.aggregate(Sum("count"))
    freight = ordersku_queryset.first().sku.spu.shipping_price
    total = sum([Decimal(ordersku.sku_json['price']) * ordersku.count for ordersku in ordersku_queryset])
    total_price = total + freight
    return {
        **count__sum,
        'freight': freight,
        'total': total_price,
        'total_price': ordersku_queryset.first().order.total_price
    }


@register.simple_tag
def ordersku(ordersku_queryset):
    return ordersku_func(ordersku_queryset)


@register.inclusion_tag('shop/member/action.html')
def order_action(order):
    ordersku_queryset = order.baykeshopordersku_set.all()
    context = ordersku_func(ordersku_queryset)
    context['order'] = order
    return context


@register.simple_tag
def comments_score(spu):
    comments = BaykeComment.objects.filter(tag=str(spu.id))
    gte_3 = comments.filter(score__gte=3).count()
    rate = gte_3 / comments.count() if comments.count() else 0.98
    score_avg = comments.aggregate(Avg('score')).get('score__avg') or 4.8
    return {
        'rate': rate * 100,
        'score_avg': score_avg
    }


'''
@file            :views.py
@Description     :通用接口
@Date            :2023/09/14 10:35:01
@Author          :幸福关中 && 轻编程
@version         :v1.0
@EMAIL           :1158920674@qq.com
@WX              :baywanyun
'''

from django.contrib import messages
from django.urls import reverse
from rest_framework.views import APIView
from rest_framework.permissions import IsAuthenticated
from rest_framework.authentication import SessionAuthentication
from rest_framework.response import Response

from baykeshop.pay.alipay.trade_page_pay import trade_page_pay
from baykeshop.common.utils import generate_order_sn
from .serializers import (
    BaykeUserUpdateAvatarSerializer, SendEmailSerializer,
    VerifyEmailSerializer, BaykeUserUpdateAboutSerializer,
    BaykeUserBalancePushSerializer
)


class BaykeUserUpdateAvatarAPIView(APIView):
    """ 修改个人头像 """
    permission_classes = [IsAuthenticated]
    authentication_classes = [SessionAuthentication]

    def post(self, request, *args, **kwargs):
        serializer = self.get_serializer()
        serializer.is_valid(raise_exception=True)
        self.perform_update(serializer)
        return Response({'code':'ok', 'avatar': request.user.baykeuser.avatar.url})
    
    def perform_update(self, serializer):
        serializer.create(serializer.validated_data)
        messages.success(self.request, message="修改成功！")

    def get_serializer(self):
        serializer = BaykeUserUpdateAvatarSerializer(
            data=self.request.data,
            context={"request": self.request}
        )
        return serializer
    

class BaykeUserUpdateAboutAPIView(BaykeUserUpdateAvatarAPIView):
    """ 修改用户简介 """
    def get_serializer(self):
        serializer = BaykeUserUpdateAboutSerializer(
            data=self.request.data,
            context={"request": self.request}
        )
        return serializer


class SendEmailAPIView(APIView):
    """ 发送邮件接口 """
    def post(self, request, *args, **kwargs):
        serializer = SendEmailSerializer(data=request.data, context={'request': request})
        serializer.is_valid(raise_exception=True)
        serializer.push_mail(serializer.validated_data)
        return Response({'code':'ok', 'message': '发送成功！'})
    

class UserUpdateEmailAPIView(APIView):
    """ 修改个人邮箱 """
    permission_classes = [IsAuthenticated]
    authentication_classes = [SessionAuthentication]

    def post(self, request, *args, **kwargs):
        serializer = VerifyEmailSerializer(data=request.data, context={'request': request})
        serializer.is_valid(raise_exception=True)
        user = request.user
        user.email = serializer.validated_data['email']
        user.save()
        messages.success(request, "修改成功！")
        return Response({'code':'ok', 'message': '修改成功！'})
    

class BaykeUserBanlancePushAPIView(APIView):
    """ 个人余额充值 """
    authentication_classes = [SessionAuthentication]
    permission_classes = [IsAuthenticated]

    def post(self, request, *args, **kwargs):
        serializer = BaykeUserBalancePushSerializer(
            data=request.data, 
            context={'request': request.user}
        )
        serializer.is_valid(raise_exception=True)
        return_url = f"{request.scheme}://{request.get_host()}{reverse('shop:balance')}"
        notify_url = f"{request.scheme}://{request.get_host()}{reverse('shop:balance')}"
        response = trade_page_pay(
            out_trade_no=generate_order_sn(request.user), 
            total_amount=serializer.validated_data['add_balance'].to_eng_string(),
            subject="个人中心充值", body=f"个人中心充值{serializer.validated_data['add_balance'].to_eng_string()}",
            return_url=return_url, notify_url=notify_url
        )
        return Response({'code': 'ok', 'payurl': response})
    


from django.db.models.signals import post_save
from django.dispatch import receiver

from baykeshop.apps.shop.models import BaykeShopOrderSKU


@receiver(post_save, sender=BaykeShopOrderSKU)
def sku_stock_sales_update(sender, instance, **kwargs):
    """ 订单关联商品保存成功 减库存 加销量 """
    from django.db.models import F
    sku = instance.sku
    sku.stock = F("stock") - instance.count
    sku.sales = F("sales") + instance.count
    sku.save()


from django.apps import AppConfig


class ShopConfig(AppConfig):
    default_auto_field = 'django.db.models.BigAutoField'
    name = 'baykeshop.apps.shop'
    verbose_name = '商品'
    
    def ready(self) -> None:
        from . import signals


from django.db import models
from django.utils.functional import cached_property
from django.utils.translation import gettext_lazy as _
# Create your models here.
from tinymce.fields import TinyMCEField
from baykeshop.common.models import BaseModelMixin


class BaykeArticleTags(BaseModelMixin):
    """Model definition for BaykeArticleTags."""
    name = models.CharField(_("名称"), max_length=50)

    # TODO: Define fields here

    class Meta:
        """Meta definition for BaykeArticleTags."""
        ordering = ['-add_date']
        verbose_name = _('文章标签')
        verbose_name_plural = verbose_name

    def __str__(self):
        """Unicode representation of BaykeArticleTags."""
        return self.name


class BaykeArticleCategory(BaseModelMixin):
    """Model definition for BaykeArticleCategory."""
    name = models.CharField(_("名称"), max_length=50)
    desc = models.CharField(_("描述"), max_length=150, blank=True, default="")
    keywords = models.CharField(_("关键词"), max_length=150, blank=True, default="")
    parent = models.ForeignKey(
        "self", 
        on_delete=models.CASCADE, 
        blank=True,
        null=True,
        verbose_name=_("父类")
    )
    status = models.BooleanField(default=True, verbose_name=_("状态"))
    sort = models.PositiveSmallIntegerField(default=1, verbose_name=_("排序"))

    # TODO: Define fields here

    class Meta:
        """Meta definition for BaykeArticleCategory."""
        ordering = ['sort']
        verbose_name = _('文章分类')
        verbose_name_plural = verbose_name

    def __str__(self):
        """Unicode representation of BaykeArticleCategory."""
        return self.name


class BaykeArticleContent(BaseModelMixin):
    """Model definition for BaykeArticleContent."""

    class ContentStatusChoices(models.IntegerChoices):
        DRAFT = 0, _("草稿")
        PUBLISH = 1, _("发布")

    title = models.CharField(_("标题"), max_length=100)
    desc = models.CharField(_("描述"), max_length=150, blank=True, default="")
    keywords = models.CharField(_("关键词"), max_length=150, blank=True, default="")
    category = models.ForeignKey(
        BaykeArticleCategory, 
        on_delete=models.CASCADE, 
        verbose_name=_("分类")
    )
    content = TinyMCEField(verbose_name=_("内容"))
    tags = models.ManyToManyField(BaykeArticleTags, blank=True, verbose_name=_("标签"))
    status = models.PositiveSmallIntegerField(
        choices=ContentStatusChoices.choices,
        default=1,
        verbose_name=_("状态")
    )

    # TODO: Define fields here

    class Meta:
        """Meta definition for BaykeArticleContent."""
        ordering = ['-pub_date']
        verbose_name = _('文章内容')
        verbose_name_plural = verbose_name

    def __str__(self):
        """Unicode representation of BaykeArticleContent."""
        return self.title

    def save(self, *args, **kwargs):
        from django.utils.html import strip_tags
        if not self.desc:
            self.desc = strip_tags(self.content)[:140]
        super().save(*args, **kwargs)

    @cached_property
    def next_article(self):
        # 下一篇
        try:
            return self.get_next_by_add_date()
        except BaykeArticleContent.DoesNotExist:
            return BaykeArticleContent.objects.last()
    
    @cached_property
    def previous_article(self):
        # 上一篇
        try:
            return self.get_previous_by_add_date()
        except BaykeArticleContent.DoesNotExist:
            return BaykeArticleContent.objects.first()





# Generated by Django 4.2.4 on 2023-11-04 13:33

import baykeshop.common.validators
from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion
import tinymce.fields


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='BaykeShopBrand',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('add_date', models.DateTimeField(auto_now_add=True, verbose_name='添加时间')),
                ('pub_date', models.DateTimeField(auto_now=True, verbose_name='修改时间')),
                ('is_delete', models.BooleanField(default=False, editable=False, verbose_name='删除标记')),
                ('name', models.CharField(max_length=100, verbose_name='品牌名称')),
                ('desc', models.CharField(blank=True, default='', max_length=150, verbose_name='品牌描述')),
                ('logo', models.ImageField(blank=True, null=True, upload_to='brand_logos/', verbose_name='品牌标志')),
            ],
            options={
                'verbose_name': '品牌',
                'verbose_name_plural': '品牌',
                'ordering': ['-add_date'],
            },
        ),
        migrations.CreateModel(
            name='BaykeShopCategory',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('add_date', models.DateTimeField(auto_now_add=True, verbose_name='添加时间')),
                ('pub_date', models.DateTimeField(auto_now=True, verbose_name='修改时间')),
                ('is_delete', models.BooleanField(default=False, editable=False, verbose_name='删除标记')),
                ('name', models.CharField(max_length=50, verbose_name='名称')),
                ('icon', models.ImageField(blank=True, max_length=200, null=True, upload_to='shop/category', verbose_name='图标')),
                ('figure', models.ImageField(blank=True, max_length=200, null=True, upload_to='shop/category', verbose_name='形象图')),
                ('sort', models.PositiveSmallIntegerField(default=1, verbose_name='排序')),
                ('is_nav', models.BooleanField(default=True, verbose_name='推荐')),
                ('status', models.BooleanField(default=True, verbose_name='状态')),
                ('parent', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='shop.baykeshopcategory', verbose_name='父类')),
            ],
            options={
                'verbose_name': '分类',
                'verbose_name_plural': '分类',
                'ordering': ['sort'],
            },
        ),
        migrations.CreateModel(
            name='BaykeShopOrder',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('add_date', models.DateTimeField(auto_now_add=True, verbose_name='添加时间')),
                ('pub_date', models.DateTimeField(auto_now=True, verbose_name='修改时间')),
                ('is_delete', models.BooleanField(default=False, editable=False, verbose_name='删除标记')),
                ('status', models.PositiveSmallIntegerField(choices=[(1, '待付款'), (2, '待发货'), (3, '待收货'), (4, '待评价'), (5, '已完成'), (6, '已关闭'), (7, '退款中')], default=1, verbose_name='订单状态')),
                ('paymethod', models.PositiveSmallIntegerField(blank=True, choices=[(None, '(Unknown)'), (1, '支付宝支付'), (2, '微信支付'), (3, '余额支付')], null=True, verbose_name='支付方式')),
                ('order_sn', models.CharField(blank=True, max_length=100, verbose_name='订单号')),
                ('total_price', models.DecimalField(blank=True, decimal_places=2, default=0, max_digits=10, verbose_name='总价')),
                ('mark', models.CharField(blank=True, default='', max_length=150, verbose_name='订单备注')),
                ('name', models.CharField(blank=True, default='', max_length=50, verbose_name='签收人')),
                ('phone', models.CharField(blank=True, default='', max_length=11, validators=[baykeshop.common.validators.validate_phone], verbose_name='手机号')),
                ('email', models.EmailField(blank=True, default='', max_length=50, verbose_name='邮箱')),
                ('address', models.CharField(blank=True, default='', max_length=200, verbose_name='收货地址')),
                ('pay_time', models.DateTimeField(blank=True, editable=False, help_text='支付时间', null=True, verbose_name='支付时间')),
                ('owner', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL, verbose_name='用户')),
            ],
            options={
                'verbose_name': '订单',
                'verbose_name_plural': '订单',
                'ordering': ['-add_date'],
                'permissions': [('send_out_goods', 'send out goods')],
            },
        ),
        migrations.CreateModel(
            name='BaykeShopSpec',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('add_date', models.DateTimeField(auto_now_add=True, verbose_name='添加时间')),
                ('pub_date', models.DateTimeField(auto_now=True, verbose_name='修改时间')),
                ('is_delete', models.BooleanField(default=False, editable=False, verbose_name='删除标记')),
                ('name', models.CharField(max_length=50, verbose_name='名称')),
            ],
            options={
                'verbose_name': '规格',
                'verbose_name_plural': '规格',
                'ordering': ['-add_date'],
            },
        ),
        migrations.CreateModel(
            name='BaykeShopSPU',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('add_date', models.DateTimeField(auto_now_add=True, verbose_name='添加时间')),
                ('pub_date', models.DateTimeField(auto_now=True, verbose_name='修改时间')),
                ('is_delete', models.BooleanField(default=False, editable=False, verbose_name='删除标记')),
                ('title', models.CharField(max_length=100, verbose_name='标题')),
                ('subtitle', models.CharField(blank=True, default='', max_length=150, verbose_name='副标题')),
                ('content', tinymce.fields.TinyMCEField(verbose_name='详情')),
                ('unit', models.CharField(blank=True, default='', max_length=50, verbose_name='单位')),
                ('shipping_price', models.DecimalField(blank=True, decimal_places=2, default=0, max_digits=10, verbose_name='运费')),
                ('status', models.BooleanField(default=True, verbose_name='状态')),
                ('brand', models.ForeignKey(on_delete=django.db.models.deletion.DO_NOTHING, to='shop.baykeshopbrand', verbose_name='商品品牌')),
                ('category', models.ManyToManyField(blank=True, to='shop.baykeshopcategory', verbose_name='商品分类')),
            ],
            options={
                'verbose_name': '商品',
                'verbose_name_plural': '商品',
                'ordering': ['-add_date'],
            },
        ),
        migrations.CreateModel(
            name='BaykeShopSPUAtlas',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('add_date', models.DateTimeField(auto_now_add=True, verbose_name='添加时间')),
                ('pub_date', models.DateTimeField(auto_now=True, verbose_name='修改时间')),
                ('is_delete', models.BooleanField(default=False, editable=False, verbose_name='删除标记')),
                ('img', models.ImageField(height_field='width', max_length=200, upload_to='spu/%Y/%m', verbose_name='商品轮播图', width_field='height')),
                ('width', models.PositiveSmallIntegerField(blank=True, default=500, verbose_name='图片宽度')),
                ('height', models.PositiveSmallIntegerField(blank=True, default=500, verbose_name='图片高度')),
                ('sort', models.PositiveSmallIntegerField(default=1, verbose_name='排序')),
                ('status', models.BooleanField(default=True, verbose_name='状态')),
                ('spu', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='shop.baykeshopspu', verbose_name='商品')),
            ],
            options={
                'verbose_name': '商品轮播图',
                'verbose_name_plural': '商品轮播图',
                'ordering': ['sort'],
            },
        ),
        migrations.CreateModel(
            name='BaykeShopSpecValue',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('add_date', models.DateTimeField(auto_now_add=True, verbose_name='添加时间')),
                ('pub_date', models.DateTimeField(auto_now=True, verbose_name='修改时间')),
                ('is_delete', models.BooleanField(default=False, editable=False, verbose_name='删除标记')),
                ('value', models.CharField(max_length=50, verbose_name='规格值')),
                ('spec', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='shop.baykeshopspec', verbose_name='规格')),
            ],
            options={
                'verbose_name': '规格值',
                'verbose_name_plural': '规格值',
                'ordering': ['add_date'],
            },
        ),
        migrations.CreateModel(
            name='BaykeShopSKU',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('add_date', models.DateTimeField(auto_now_add=True, verbose_name='添加时间')),
                ('pub_date', models.DateTimeField(auto_now=True, verbose_name='修改时间')),
                ('is_delete', models.BooleanField(default=False, editable=False, verbose_name='删除标记')),
                ('img', models.ImageField(max_length=200, upload_to='shop/sku', verbose_name='主图')),
                ('price', models.DecimalField(decimal_places=2, max_digits=10, verbose_name='售价')),
                ('cost_price', models.DecimalField(decimal_places=2, max_digits=10, verbose_name='成本价')),
                ('discount_price', models.DecimalField(decimal_places=2, max_digits=10, verbose_name='划线价')),
                ('stock', models.PositiveSmallIntegerField(default=0, verbose_name='库存')),
                ('sales', models.PositiveSmallIntegerField(default=0, verbose_name='销量')),
                ('code', models.CharField(blank=True, default='', max_length=20, verbose_name='商品编码')),
                ('volume', models.DecimalField(blank=True, decimal_places=2, default=0, max_digits=10, verbose_name='体积')),
                ('weight', models.DecimalField(blank=True, decimal_places=2, default=0, max_digits=10, verbose_name='重量')),
                ('specs', models.JSONField(blank=True, default=dict, verbose_name='规格数据')),
                ('spu', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='shop.baykeshopspu', verbose_name='SPU')),
            ],
            options={
                'verbose_name': '商品规格',
                'verbose_name_plural': '商品规格',
                'ordering': ['price'],
            },
        ),
        migrations.CreateModel(
            name='BaykeShopOrderSKU',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('add_date', models.DateTimeField(auto_now_add=True, verbose_name='添加时间')),
                ('pub_date', models.DateTimeField(auto_now=True, verbose_name='修改时间')),
                ('is_delete', models.BooleanField(default=False, editable=False, verbose_name='删除标记')),
                ('count', models.PositiveSmallIntegerField(default=1, verbose_name='数量')),
                ('sku_json', models.JSONField(blank=True, default=dict, verbose_name='商品快照')),
                ('is_commented', models.BooleanField(default=False, verbose_name='是否已评价')),
                ('order', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='shop.baykeshoporder', verbose_name='订单')),
                ('sku', models.ForeignKey(on_delete=django.db.models.deletion.PROTECT, to='shop.baykeshopsku', verbose_name='商品规格')),
            ],
            options={
                'verbose_name': '订单商品',
                'verbose_name_plural': '订单商品',
                'ordering': ['-add_date'],
            },
        ),
        migrations.CreateModel(
            name='BaykeShopCart',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('add_date', models.DateTimeField(auto_now_add=True, verbose_name='添加时间')),
                ('pub_date', models.DateTimeField(auto_now=True, verbose_name='修改时间')),
                ('is_delete', models.BooleanField(default=False, editable=False, verbose_name='删除标记')),
                ('num', models.PositiveSmallIntegerField(default=1, verbose_name='数量')),
                ('owner', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL, verbose_name='用户')),
                ('sku', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='shop.baykeshopsku', verbose_name='规格')),
            ],
            options={
                'verbose_name': '购物车',
                'verbose_name_plural': '购物车',
                'ordering': ['-add_date'],
            },
        ),
        migrations.CreateModel(
            name='BaykeAddress',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('add_date', models.DateTimeField(auto_now_add=True, verbose_name='添加时间')),
                ('pub_date', models.DateTimeField(auto_now=True, verbose_name='修改时间')),
                ('is_delete', models.BooleanField(default=False, editable=False, verbose_name='删除标记')),
                ('name', models.CharField(max_length=50, verbose_name='签收人')),
                ('phone', models.CharField(max_length=11, verbose_name='手机号')),
                ('email', models.EmailField(blank=True, default='', max_length=50, verbose_name='邮箱')),
                ('province', models.CharField(max_length=150, verbose_name='省')),
                ('city', models.CharField(max_length=150, verbose_name='市')),
                ('county', models.CharField(max_length=150, verbose_name='区/县')),
                ('address', models.CharField(max_length=150, verbose_name='详细地址')),
                ('is_default', models.BooleanField(default=False, verbose_name='设为默认')),
                ('owner', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL, verbose_name='用户')),
            ],
            options={
                'verbose_name': '收货地址',
                'verbose_name_plural': '收货地址',
                'ordering': ['-add_date'],
            },
        ),
        migrations.AddConstraint(
            model_name='baykeshoporder',
            constraint=models.UniqueConstraint(models.F('owner'), models.F('order_sn'), name='unique_owner_order'),
        ),
        migrations.AddConstraint(
            model_name='baykeshopcart',
            constraint=models.UniqueConstraint(models.F('owner'), models.F('sku'), name='unique_owner_sku'),
        ),
    ]



#!/usr/bin/env python
# -*- encoding: utf-8 -*-
'''
@文件    :__init__.py
@说明    :项目配置
@时间    :2023/04/22 11:07:06
@作者    :幸福关中&轻编程
@版本    :1.0
@微信    :baywanyun
'''

from django.conf import settings
from . import default


class Settings:
    
    def __init__(self, conf=None) -> None:
        self._conf = conf or default.DEFAULTS_CONF
        
    def __getattribute__(self, __name: str) -> str:
        try:
            return super().__getattribute__(__name)
        except AttributeError:
            return getattr(settings, __name.upper(), self.get_attr(__name)) 
            
    def get_attr(self, name):
        try:
            return self._conf[name.upper()]
        except KeyError:
            raise ValueError("该配置项未设置...")
    
bayke_settings = Settings()


# Generated by Django 4.2.4 on 2023-11-04 13:33

from django.db import migrations, models
import django.db.models.deletion
import tinymce.fields


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='BaykeArticleCategory',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('add_date', models.DateTimeField(auto_now_add=True, verbose_name='添加时间')),
                ('pub_date', models.DateTimeField(auto_now=True, verbose_name='修改时间')),
                ('is_delete', models.BooleanField(default=False, editable=False, verbose_name='删除标记')),
                ('name', models.CharField(max_length=50, verbose_name='名称')),
                ('desc', models.CharField(blank=True, default='', max_length=150, verbose_name='描述')),
                ('keywords', models.CharField(blank=True, default='', max_length=150, verbose_name='关键词')),
                ('status', models.BooleanField(default=True, verbose_name='状态')),
                ('sort', models.PositiveSmallIntegerField(default=1, verbose_name='排序')),
                ('parent', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='article.baykearticlecategory', verbose_name='父类')),
            ],
            options={
                'verbose_name': '文章分类',
                'verbose_name_plural': '文章分类',
                'ordering': ['sort'],
            },
        ),
        migrations.CreateModel(
            name='BaykeArticleTags',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('add_date', models.DateTimeField(auto_now_add=True, verbose_name='添加时间')),
                ('pub_date', models.DateTimeField(auto_now=True, verbose_name='修改时间')),
                ('is_delete', models.BooleanField(default=False, editable=False, verbose_name='删除标记')),
                ('name', models.CharField(max_length=50, verbose_name='名称')),
            ],
            options={
                'verbose_name': '文章标签',
                'verbose_name_plural': '文章标签',
                'ordering': ['-add_date'],
            },
        ),
        migrations.CreateModel(
            name='BaykeArticleContent',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('add_date', models.DateTimeField(auto_now_add=True, verbose_name='添加时间')),
                ('pub_date', models.DateTimeField(auto_now=True, verbose_name='修改时间')),
                ('is_delete', models.BooleanField(default=False, editable=False, verbose_name='删除标记')),
                ('title', models.CharField(max_length=100, verbose_name='标题')),
                ('desc', models.CharField(blank=True, default='', max_length=150, verbose_name='描述')),
                ('keywords', models.CharField(blank=True, default='', max_length=150, verbose_name='关键词')),
                ('content', tinymce.fields.TinyMCEField(verbose_name='内容')),
                ('status', models.PositiveSmallIntegerField(choices=[(0, '草稿'), (1, '发布')], default=1, verbose_name='状态')),
                ('category', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='article.baykearticlecategory', verbose_name='分类')),
                ('tags', models.ManyToManyField(blank=True, to='article.baykearticletags', verbose_name='标签')),
            ],
            options={
                'verbose_name': '文章内容',
                'verbose_name_plural': '文章内容',
                'ordering': ['-pub_date'],
            },
        ),
    ]



'''
@file            :views.py
@Description     :接口相关视图
@Date            :2023/09/11 21:03:47
@Author          :幸福关中 && 轻编程
@version         :v1.0
@EMAIL           :1158920674@qq.com
@WX              :baywanyun
'''

from django.contrib import messages
from rest_framework.response import Response
from rest_framework.views import APIView
from rest_framework.generics import CreateAPIView
from rest_framework import viewsets
from rest_framework.permissions import IsAuthenticated
from rest_framework.authentication import SessionAuthentication

from baykeshop.common.permission import IsOwnerAuthenticated
from baykeshop.apps.shop.models import BaykeShopCart, BaykeAddress
from .serializers import (
    BaykeShopCartSerializer, BaykeShopCartNumSerializer,
    BaykeShopCreateOrderSerializer, BaykeAddressSerializer,
    BaykeShopOrderCashSerializer, ConfirmReceiptSerializer
)


class BaykeShopCartCreateAPIView(CreateAPIView):
    """ 商品添加购物车 """
    authentication_classes = [SessionAuthentication]
    permission_classes = [IsAuthenticated]
    serializer_class = BaykeShopCartSerializer

    def get_queryset(self):
        return BaykeShopCart.objects.filter(owner=self.request.user)
    

class BaykeShopCartUpdateNumAPIView(APIView):
    """ 修改购物车商品数量 """
    authentication_classes = [SessionAuthentication]
    permission_classes = [IsAuthenticated, IsOwnerAuthenticated]
    
    def post(self, request, *args, **kwargs):
        serializer = self.get_serializer()
        serializer.is_valid(raise_exception=True)
        BaykeShopCart.objects.filter(
            id=serializer.validated_data['cartid'],
            owner=request.user
        ).update(num=serializer.validated_data['num'])
        return Response({"code": "ok"})

    def delete(self, request, *args, **kwargs):
        serializer = self.get_serializer()
        serializer.is_valid(raise_exception=True)
        BaykeShopCart.objects.filter(
            id=serializer.validated_data['cartid'],
            owner=request.user,
        ).delete()
        return Response({"code": "ok"})

    def get_serializer(self):
        serializer = BaykeShopCartNumSerializer(
            data=self.request.data, 
            context={"request": self.request}
        )
        return serializer
    

class BaykeShopOrderCreateAPIView(CreateAPIView):
    """ 创建订单 """
    authentication_classes = [SessionAuthentication]
    permission_classes = [IsAuthenticated]
    serializer_class = BaykeShopCreateOrderSerializer


class BaykeShopOrderCashAPIView(APIView):
    """ 立即支付 """
    authentication_classes = [SessionAuthentication]
    permission_classes = [IsAuthenticated]

    def post(self, request, *args, **kwargs):
        serializer = BaykeShopOrderCashSerializer(
            data=request.data, 
            context={"request": self.request}
        )
        serializer.is_valid(raise_exception=True)
        instance=serializer.get_instance(serializer.validated_data['orderid'])
        serializer.update(instance, serializer.validated_data)
        return Response(serializer.data)


class BaykeAddressViewSet(viewsets.ModelViewSet):
    """ 地址增删改查 """
    serializer_class = BaykeAddressSerializer
    permission_classes = [IsAuthenticated, IsOwnerAuthenticated]

    def get_queryset(self):
        return BaykeAddress.objects.filter(owner=self.request.user)
    
    def perform_create(self, serializer):
        self.save_only_default(serializer)
        return super().perform_create(serializer)
    
    def perform_update(self, serializer):
        self.save_only_default(serializer)
        return super().perform_update(serializer)
    
    def save_only_default(self, serializer):
        # 处理默认收货地址只能有一个
        if serializer.validated_data['is_default']:
            self.get_queryset().filter(is_default=True).update(is_default=False)


class ConfirmReceiptAPIView(APIView):
    """ 确认收货接口 """
    authentication_classes = [SessionAuthentication]
    permission_classes = [IsAuthenticated]

    def post(self, request, *args, **kwargs):
        serializer = ConfirmReceiptSerializer(data=request.data, context={'request': request})
        serializer.is_valid(raise_exception=True)
        order = serializer.get_order(serializer.validated_data['orderid'])
        order.update(status=4)
        messages.success(request, '确认收货成功！')
        return Response({'code':'ok'})


'''
@file            :widgets.py
@Description     :自定义Widget
@Date            :2023/09/03 15:26:17
@Author          :幸福关中 && 轻编程
@version         :v1.0
@EMAIL           :1158920674@qq.com
@WX              :baywanyun
'''

from django.forms import widgets
from captcha.fields import CaptchaTextInput as BaseCaptchaTextInput


class TextInput(widgets.TextInput):
    input_type = "text"
    template_name = "system/widgets/text.html"


class PasswordInput(widgets.PasswordInput):
    input_type = "password"
    template_name = "system/widgets/text.html"


class CaptchaTextInput(BaseCaptchaTextInput):
    template_name = "system/widgets/captcha.html"




#!/usr/bin/env python
# -*- encoding: utf-8 -*-
'''
@文件    :permission.py
@说明    :权限相关公共类
@时间    :2023/10/31 10:02:53
@作者    :幸福关中&轻编程
@版本    :1.0
@微信    :baywanyun
'''


from rest_framework import permissions

class IsOwnerAuthenticated(permissions.IsAuthenticated):
    """ 仅拥有获取自己个人相关信息的权限 """
    
    def has_permission(self, request, view):
        return super().has_permission(request, view)
    
    def has_object_permission(self, request, view, obj):
        return bool(request.user == obj.owner)


#!/usr/bin/env python
# -*- encoding: utf-8 -*-
'''
@文件    :signals.py
@说明    :信号
@时间    :2023/06/27 23:51:31
@作者    :幸福关中&轻编程
@版本    :1.0
@微信    :baywanyun
'''
from django.contrib.auth import get_user_model
from django.dispatch import receiver
from django.db.models.signals import post_save

from .models import BaykeUser


@receiver(post_save, sender=get_user_model())
def save_user_handler(sender, instance, created, **kwargs):
    # 同步创建一个用户的拓展字段
    if created:
        try:
            instance.baykeuser
        except BaykeUser.DoesNotExist:
            user = BaykeUser(
                owner=instance, 
                name=instance.username
            )
            user.save()

    


'''
@file            :forms.py
@Description     :表单类
@Date            :2023/09/06 11:02:42
@Author          :幸福关中 && 轻编程
@version         :v1.0
@EMAIL           :1158920674@qq.com
@WX              :baywanyun
'''

from django import forms
from django.forms import widgets
from django.contrib.admin.widgets import FilteredSelectMultiple

from .models import BaykeShopSKU, BaykeShopSpecValue


class SpecsFilteredSelectMultiple(FilteredSelectMultiple):

    def get_context(self, name, value, attrs):
        return super().get_context(name, value, attrs)


class BaykeShopSKUForm(forms.ModelForm):
    """ sku表单 """
    specs = forms.ModelMultipleChoiceField(
        queryset=BaykeShopSpecValue.objects.all(),
        widget=SpecsFilteredSelectMultiple(verbose_name="规格", is_stacked=False),
        label="规格",
        required=False,
        help_text="备注：如果为单规格商品则不必勾选，如果存在多个SKU，则必须选择"
    )

    class Meta:
        model = BaykeShopSKU
        fields = "__all__"

    def clean_specs(self):
        value_dict = {}
        specs = self.cleaned_data.get('specs')
        from collections import OrderedDict
        # 转换为有序字典
        data = specs.values('spec__name')
        data_dict = {}
        for item in data:
            data_dict[frozenset(item.items())] = item
        # 去重
        unique_data = list(OrderedDict.fromkeys(data_dict.keys()))
        # 去重后的长度小于初始长度说明存在重复
        if len(unique_data) < len(data):
            raise forms.ValidationError("一个sku不能有重复的规格，例如：一件衣服不可能既是绿色又是红色")
        
        for op in specs:
            item_dict = {}
            item_dict[op.id] = op.value
            item_dict[op.spec.name] = op.value
            value_dict.update(item_dict)
        return value_dict



from django.db import models
from django.utils.translation import gettext_lazy as _
from django.contrib.auth import get_user_model
from django.contrib.auth.models import Permission
# Create your models here.
from baykeshop.common.models import BaseModelMixin, ContentTypeAbstract


class BaykeADPosition(BaseModelMixin):
    """Model definition for BaykeADPosition."""
    name = models.CharField(_("名称"), max_length=50)
    slug = models.SlugField(_("slug"), unique=True)
    desc = models.CharField(_("说明"), max_length=150, blank=True, default="")
    # TODO: Define fields here

    class Meta:
        """Meta definition for BaykeADPosition."""
        ordering = ['-add_date']
        verbose_name = _("广告位")
        verbose_name_plural = verbose_name

    def __str__(self):
        """Unicode representation of BaykeADPosition."""
        return self.desc or self.name
    
    @classmethod
    def get_position_spaces(cls, slug):
        spaces = []
        try:
            position = cls.objects.get(slug=slug)
            spaces = position.baykeadspace_set.exclude(status=False)
        except cls.DoesNotExist:
            pass
        return spaces


class BaykeADSpace(BaseModelMixin):
    """Model definition for BaykeADSpace."""

    class ADSpaceChoices(models.TextChoices):
        TEXT = "text", _("文字广告")
        IMG = "img", _("图片广告")
        HTML = "html", _("自定义广告")

    name = models.CharField(_("名称"), max_length=50)
    slug = models.SlugField(_("slug"), max_length=50, unique=True)
    space = models.CharField(
        choices=ADSpaceChoices.choices, 
        max_length=10, 
        default="img",
        verbose_name=_("类型")
    )
    position = models.ForeignKey(
        BaykeADPosition, 
        on_delete=models.CASCADE, 
        verbose_name=_("广告位"),
        blank=True,
        null=True
    )
    remark = models.CharField(_("备注"), max_length=150, blank=True, default="")
    img = models.ImageField(_("图片广告"), upload_to="common/ad", blank=True, null=True)
    text = models.CharField(_("文字广告"), max_length=150, blank=True, default="")
    html = models.TextField(_("自定义广告"), blank=True, default="")
    target = models.URLField(_("跳转地址"), max_length=200, blank=True, default="")
    sort = models.PositiveSmallIntegerField(_("排序"), default=1)
    status = models.BooleanField(_("状态"), default=True)

    # TODO: Define fields here

    class Meta:
        """Meta definition for BaykeADSpace."""
        ordering = ['sort']
        verbose_name = _("广告内容")
        verbose_name_plural = verbose_name

    def __str__(self):
        """Unicode representation of BaykeADSpace."""
        return self.name
    
    @classmethod
    def get_space(cls, slug):
        obj = None
        try:
            obj = cls.objects.get(slug=slug)
        except cls.DoesNotExist:
            pass
        return obj
    

class BaykeComment(ContentTypeAbstract):
    """Model definition for BaykeComment."""

    class ScoreChoices(models.IntegerChoices):
        BAD = 1, _("差评")
        MIDDL = 3, _("中评")
        GOOD = 5, _("好评")

    owner = models.ForeignKey(
        get_user_model(), 
        on_delete=models.CASCADE, 
        verbose_name=_("用户")
    )
    content = models.CharField(_("评论"), max_length=200)
    score = models.PositiveSmallIntegerField(
        choices=ScoreChoices.choices, 
        default=5,
        verbose_name=_("评价")
    )
    reply = models.CharField(_("回复内容"), max_length=200, blank=True, default="")
    # TODO: Define fields here

    class Meta:
        """Meta definition for BaykeComment."""
        ordering = ['-add_date']
        verbose_name = _("评论")
        verbose_name_plural = verbose_name
        permissions = [
            ("reply_to_comments", "Reply to comments"),
        ]

    def __str__(self):
        """Unicode representation of BaykeComment."""
        return self.content
    

class BaykeSiteMenus(BaseModelMixin):
    """ 后台管理菜单自定义 """
    name = models.CharField(_("名称"), max_length=50, unique=True)
    parent = models.ForeignKey("self", on_delete=models.CASCADE, blank=True, null=True)
    icon = models.CharField(_("图标"), max_length=50, blank=True, default="")
    permission = models.OneToOneField(
        Permission, 
        on_delete=models.CASCADE, 
        blank=True, 
        null=True,
        verbose_name=_("权限")
    )

    # TODO: Define fields here

    class Meta:
        ordering = ['-add_date']
        verbose_name = 'SiteMenus'
        verbose_name_plural = 'SiteMenus'

    def __str__(self):
        return self.name


#!/usr/bin/env python
# -*- encoding: utf-8 -*-
'''
@文件    :renderers.py
@说明    :渲染相关公共类
@时间    :2023/10/31 10:03:23
@作者    :幸福关中&轻编程
@版本    :1.0
@微信    :baywanyun
'''


from rest_framework import renderers
from rest_framework.utils.serializer_helpers import ReturnList
from django.core.serializers.json import DjangoJSONEncoder


class JSONRenderer(renderers.JSONRenderer):
    
    def render(self, data, accepted_media_type=None, renderer_context=None):
        return super().render(data, accepted_media_type, renderer_context)
    

class TemplateHTMLRenderer(renderers.TemplateHTMLRenderer):
    
    def render(self, data, accepted_media_type=None, renderer_context=None):
        if isinstance(data, ReturnList):
            data = {"results": data}
        return super().render(data, accepted_media_type, renderer_context)


from django.urls import path, include
from . import views

app_name = 'system'

urlpatterns = [
    path('login/', views.LoginView.as_view(), name='login'),
    path('logout/', views.LogoutView.as_view(), name='logout'),
    path('register/', views.RegisterView.as_view(), name='register'),
    path('api/', include('baykeshop.apps.system.api.urls'))
]


'''
@file            :serializers.py
@Description     :接口序列化
@Date            :2023/09/11 21:01:30
@Author          :幸福关中 && 轻编程
@version         :v1.0
@EMAIL           :1158920674@qq.com
@WX              :baywanyun
'''

from django.db.models import F
from django.db.utils import IntegrityError
from django.urls import reverse
from django.utils.translation import gettext_lazy as _
from django.contrib import messages
from django.utils import timezone

from rest_framework import serializers

from baykeshop.common.validators import validate_phone
from baykeshop.apps.shop.models import (
    BaykeShopCart, BaykeShopOrder, BaykeShopOrderSKU,
    BaykeAddress
)
from baykeshop.apps.user.models import BaykeUserBalanceLog
from baykeshop.pay.alipay.client import client
from baykeshop.pay.alipay.trade_page_pay import trade_page_pay


class BaykeShopCartSerializer(serializers.ModelSerializer):
    """ 购物车序列化 """
    owner = serializers.HiddenField(default=serializers.CurrentUserDefault())

    class Meta:
        model = BaykeShopCart
        fields = "__all__"
    
    def create(self, validated_data):
        try:
            instance = super().create(validated_data)
        except IntegrityError:
            carts = BaykeShopCart.objects.filter(
                owner=self.context['request'].user, 
                sku=validated_data.get('sku')
            )
            if carts.exists():
                carts.update(num=F("num")+validated_data["num"])
                instance = carts.first()
        return instance


class BaykeShopCartNumSerializer(serializers.Serializer):
    """ 购物车商品数量修改接口 """
    cartid = serializers.IntegerField(min_value=1)
    num = serializers.IntegerField(min_value=1, required=False, default=1)

    def validate_cartid(self, cartid):
        try:
            self.get_instance()
        except BaykeShopCart.DoesNotExist:
            raise serializers.ValidationError("该购物车商品不存在，请检查！")
        return cartid
    
    def validate_num(self, num):
        instance = self.get_instance()
        if instance.sku.stock < self.initial_data['num']:
            raise serializers.ValidationError("数量不能大于库存！")
        return num

    def get_instance(self):
        initial_data = self.initial_data
        return BaykeShopCart.objects.get(
            id=initial_data['cartid'], 
            owner=self.context['request'].user
        )
    

class BaykeShopOrderSKUSerializer(serializers.ModelSerializer):
    """ 订单商品 """
    class Meta:
        model = BaykeShopOrderSKU
        exclude = ('order',)

class BaykeShopCreateOrderSerializer(serializers.ModelSerializer):
    """ 订单 """
    owner = serializers.HiddenField(default=serializers.CurrentUserDefault())
    baykeshopordersku_set = BaykeShopOrderSKUSerializer(many=True)

    class Meta:
        model = BaykeShopOrder
        fields = "__all__"

    def validate(self, attrs):
        attrs = super().validate(attrs)
        attrs['total_price'] = self.get_total_price(attrs)
        return attrs

    def create(self, validated_data):
        baykeshopordersku_set = validated_data.pop('baykeshopordersku_set')
        instance = super().create(validated_data)
        for ordersku in baykeshopordersku_set:
            ordersku['order'] = instance
            BaykeShopOrderSKU.objects.create(**ordersku)
        return instance
    
    def get_total_price(self, attrs):
        baykeshopordersku_set = attrs['baykeshopordersku_set']
        shipping_price = baykeshopordersku_set[0]['sku'].spu.shipping_price
        total_price = sum([(ordersku['count'] * ordersku['sku'].price) 
             for ordersku in baykeshopordersku_set]) + shipping_price
        return total_price
    
    @property
    def data(self):
        data = super().data
        data['cashurl'] = reverse("shop:order-cash", args=[data['id']])
        return data
    

class BaykeShopOrderCashSerializer(serializers.Serializer):
    """ 订单结算 """
    orderid = serializers.IntegerField(min_value=1)
    paymethod = serializers.IntegerField(
        min_value=1, 
        max_value=3, 
        default=1, 
        error_messages={
            'min_value': _('不支持该支付方式.'),
            'max_value': _('不支持该支付方式.')
        }
    )
    name = serializers.CharField(min_length=1, max_length=50)
    phone = serializers.CharField(validators=[validate_phone], min_length=11, max_length=11)
    email = serializers.EmailField(required=False, min_length=3, max_length=60)
    address = serializers.CharField(min_length=10, max_length=150)
    mark = serializers.CharField(max_length=150, required=False, allow_blank=True)

    def validate_orderid(self, orderid):
        try:
            self.get_instance(orderid)
        except BaykeShopOrder.DoesNotExist:
            raise serializers.ValidationError("订单不存在或已支付，请检查！")
        return orderid

    def update(self, instance, validated_data):
        instance.name = validated_data['name']
        instance.paymethod = validated_data['paymethod']
        instance.phone = validated_data['phone']
        instance.address = validated_data['address']
        instance.mark = validated_data['mark']
        instance.status = 1 
        instance.save()
        return instance
    
    def get_instance(self, orderid):
        return BaykeShopOrder.objects.get(
            id=orderid, owner=self.context['request'].user, status=1
        )
    
    @property
    def data(self):
        data = super().data
        instance = self.get_instance(data['orderid'])
        data['payurl'] = self.get_payurl(instance)
        return data
    
    def get_payurl(self, instance):
        request = self.context['request']
        if instance.paymethod == 1:
            return_url = f"{request.scheme}://{request.get_host()}{reverse('shop:alipay')}"
            notify_url = f"{request.scheme}://{request.get_host()}{reverse('shop:alipay')}"
            response = trade_page_pay(
                out_trade_no=instance.order_sn, total_amount=instance.total_price.to_eng_string(),
                subject=instance.order_sn, body=f"alipay{instance.order_sn}", 
                return_url=return_url, notify_url=notify_url
            )
            payurl = response
            return payurl
        elif instance.paymethod == 2:
            pass
        elif instance.paymethod == 3:
            baykeuser = request.user.baykeuser
            balance = baykeuser.balance
            # 判断余额是否足够
            if balance < instance.total_price:
                raise serializers.ValidationError('余额不足')
            # 减掉用户余额
            baykeuser.balance -= instance.total_price
            baykeuser.save()
            # 修改订单状态
            instance.status = 2
            instance.paymethod = 3
            instance.paytime = timezone.now()
            instance.save()
            # 创建余额变动记录
            BaykeUserBalanceLog.objects.create(
                owner=request.user, 
                amount=instance.total_price, 
                change_status=2,
                change_way=3
            )
            messages.success(request, '支付成功，请耐心等待发货!')
            return reverse('shop:orders-detail', args=[instance.id])


class BaykeAddressSerializer(serializers.ModelSerializer):
    """ 地址序列化 """
    owner = serializers.HiddenField(default=serializers.CurrentUserDefault())
    
    class Meta:
        model = BaykeAddress
        fields = "__all__"


class ConfirmReceiptSerializer(serializers.Serializer):
    """ 确认收货 """
    orderid = serializers.IntegerField(min_value=1)

    def validate_orderid(self, orderid):
        order = self.get_order(orderid)
        if not order.exists():
            raise serializers.ValidationError("该订单不存在！")
        return orderid
    
    def get_order(self, orderid):
        order = BaykeShopOrder.objects.filter(
            id=orderid, status=3, owner=self.context['request'].user
        )
        return order


'''
@file            :views.py
@Description     :通用接口
@Date            :2023/09/14 10:35:01
@Author          :幸福关中 && 轻编程
@version         :v1.0
@EMAIL           :1158920674@qq.com
@WX              :baywanyun
'''

from django.contrib import messages
from rest_framework.views import APIView
from rest_framework.permissions import IsAuthenticated
from rest_framework.authentication import SessionAuthentication
from rest_framework.response import Response

from .serializers import BaykeShopOrderCommentSerializer


class BaykeShopOrderCommentAPIView(APIView):
    """ 留言 """
    authentication_classes = [SessionAuthentication]
    permission_classes = [IsAuthenticated]

    def post(self, request, *args, **kwargs):
        serializer = BaykeShopOrderCommentSerializer(
            data=request.data,
            context={'request':request}
        )
        serializer.is_valid(raise_exception=True)
        serializer.create(serializer.validated_data)
        messages.success(request, '评价成功！')
        return Response({'code':'ok'})


from typing import Any
from django.contrib import admin
from django.db.models.query import QuerySet
from django.http.request import HttpRequest
from django.template.loader import render_to_string
from django.template.response import TemplateResponse
from django.shortcuts import get_object_or_404, redirect
from django.contrib.admin.views.decorators import staff_member_required
from django.contrib.auth.decorators import permission_required
from django.utils.decorators import method_decorator

# Register your models here.
from baykeshop.common.options import ModelAdmin, TabularInline, StackedInline
from .models import (
    BaykeShopCategory, BaykeShopBrand, BaykeShopSPU,
    BaykeShopSKU, BaykeShopSpec, BaykeShopSpecValue,
    BaykeShopSPUAtlas, BaykeShopOrder, BaykeShopOrderSKU
)
from .forms import BaykeShopSKUForm


class BaykeShopSKUInline(StackedInline):
    '''Tabular Inline View for BaykeShopSKU'''
    model = BaykeShopSKU
    min_num = 1
    max_num = 20
    extra = 0
    form = BaykeShopSKUForm


class BaykeShopSpecValueInline(TabularInline):
    '''Tabular Inline View for BaykeShopSpecValue'''

    model = BaykeShopSpecValue
    min_num = 1
    max_num = 20
    extra = 1

class BaykeShopSPUAtlasInline(TabularInline):
    '''Stacked Inline View for BaykeShopSPUAtlas'''

    model = BaykeShopSPUAtlas
    min_num = 1
    max_num = 10
    extra = 1
    

class BaykeShopCategoryInline(TabularInline):
    '''Stacked Inline View for BaykeShopCategory'''

    model = BaykeShopCategory
    min_num = 1
    max_num = 100
    extra = 1
    exclude = ('figure', 'desc')


@admin.register(BaykeShopCategory)
class BaykeShopCategoryAdmin(ModelAdmin):
    '''Admin View for BaykeShopCategory'''

    list_display = ('id', 'name', 'parent', 'sort', 'status', 'add_date')
    list_display_links = ('id', 'name')
    inlines = [BaykeShopCategoryInline]
    exclude = ('parent', )

    def get_queryset(self, request: HttpRequest) -> QuerySet[Any]:
        return super().get_queryset(request).exclude(parent__isnull=False)


@admin.register(BaykeShopBrand)
class BaykeShopBrandAdmin(ModelAdmin):
    '''Admin View for BaykeShopBrand'''

    list_display = ('id', 'name', 'add_date')


@admin.register(BaykeShopSPU)
class BaykeShopSPUAdmin(ModelAdmin):
    '''Admin View for BaykeShopSPU'''

    list_display = ('id', 'title', 'brand', 'add_date')
    # filter_horizontal = ("category", )
    list_display_links = ('id', 'title')
    inlines = [BaykeShopSKUInline, BaykeShopSPUAtlasInline]

    def formfield_for_manytomany(self, db_field, request, **kwargs):
        if db_field.name == 'category':
            kwargs['queryset'] = BaykeShopCategory.objects.exclude(parent__isnull=True)
        return super().formfield_for_manytomany(db_field, request, **kwargs)
    
    def save_formset(self, request: Any, form: Any, formset: Any, change: Any) -> None:
        if issubclass(formset.form, BaykeShopSKUForm):
            specs = [ str(data['specs'].keys()) for data in formset.cleaned_data if data['specs']]   
            if len(set(specs)) < len(specs):
                self.message_user(request, "规格重复,不能存在两个同样规格的sku!", level="ERROR")      
        super().save_formset(request, form, formset, change)

    
@admin.register(BaykeShopSpec)
class BaykeShopSpecAdmin(ModelAdmin):
    '''Admin View for BaykeShopSpec'''

    list_display = ('id', 'name', 'add_date')
    inlines = [BaykeShopSpecValueInline,]


@admin.register(BaykeShopOrder)
class BaykeShopOrderAdmin(ModelAdmin):
    '''Admin View for BaykeShopOrder'''

    list_display = (
        'id', 'owner', 'order_sn', 'orderskus', 'status', 
        'paymethod', 'total_price', 'pay_time', 'orderaction'
    )
    list_filter = ('paymethod', 'status')
    list_display_links = ('id', 'owner', 'order_sn')
    readonly_fields = ('owner', 'order_sn', 'paymethod', 'status', 'orderskus', )

    @admin.display(description="订单商品")
    def orderskus(self, obj):
        orderskus = obj.baykeshopordersku_set.all()
        return render_to_string(
            "shop/admin/orderskus.html", 
            context={'orderskus': orderskus}
        )
    
    @admin.display(description="操作")
    def orderaction(self, obj):
        return render_to_string("shop/admin/orderaction.html", context={'obj':obj})

    def get_urls(self):
        from django.urls import path
        urls = super().get_urls()
        my_urls = [path("sendGoods/<int:pk>/", self.admin_site.admin_view(self.send_goods), name='send_goods')]
        return my_urls + urls

    @method_decorator(staff_member_required)
    @method_decorator(permission_required("shop.send_out_goods", raise_exception=True))
    def send_goods(self, request, pk):
        # 订单发货操作
        obj = get_object_or_404(BaykeShopOrder, pk=pk)
        context = dict(
            self.admin_site.each_context(request),
            title=f"订单{obj.order_sn}发货操作",
            order=obj
        )
        if request.method == 'POST':
            obj.status = 3
            obj.save()
            return redirect('admin:shop_baykeshoporder_changelist')
        return TemplateResponse(request, "shop/admin/send_goods.html", context)
    
    def has_change_permission(self, request: HttpRequest, obj=None) -> bool:
        if obj and obj.status == 1:
            return True
        return False
    
    def has_add_permission(self, request: HttpRequest) -> bool:
        return False


# @admin.register(BaykeShopOrderSKU)
# class BaykeShopOrderSKUAdmin(ModelAdmin):
#     '''Admin View for BaykeShopOrderSKU'''

#     list_display = ('id', 'order', 'sku', 'count')
   
   


'''
@file            :urls.py
@Description     :通用接口url
@Date            :2023/09/14 10:42:37
@Author          :幸福关中 && 轻编程
@version         :v1.0
@EMAIL           :1158920674@qq.com
@WX              :baywanyun
'''

from django.urls import path

from . import views

app_name = "userapi"

urlpatterns = [
    path(
        'update-user-avatar/', 
         views.BaykeUserUpdateAvatarAPIView.as_view(), 
         name='update-user-avatar'
    ),
    path(
        'update-user-about/', 
         views.BaykeUserUpdateAboutAPIView.as_view(), 
         name='update-user-about'
    ),
    path(
        'send-email/', 
         views.SendEmailAPIView.as_view(), 
         name='send-email'
    ),
    path(
        'update-user-email/', 
         views.UserUpdateEmailAPIView.as_view(), 
         name='update-user-email'
    ),
    path(
        'push-balance/', 
         views.BaykeUserBanlancePushAPIView.as_view(), 
         name='push-balance'
    )
]




'''
@file            :serializers.py
@Description     :通用接口序列化
@Date            :2023/09/14 10:29:32
@Author          :幸福关中 && 轻编程
@version         :v1.0
@EMAIL           :1158920674@qq.com
@WX              :baywanyun
'''
from django.contrib.auth import get_user_model
from django.core.cache import cache
from rest_framework import serializers

from baykeshop.conf import bayke_settings
from baykeshop.common.utils import code_random, push_main
from baykeshop.apps.user.models import BaykeUser

class BaykeUserUpdateAvatarSerializer(serializers.ModelSerializer):
    """ 修改用户头像 """
    class Meta:
        model = BaykeUser
        fields = ('avatar', )

    def create(self, validated_data):
        user = self.context['request'].user
        try:
            baykeuser = user.baykeuser
        except BaykeUser.DoesNotExist:
            baykeuser = BaykeUser.objects.create(owner=user, name=user.username)
        self.update(baykeuser, validated_data)
        return baykeuser
    
    def update(self, instance, validated_data):
        instance.avatar = validated_data['avatar']
        instance.save()
        return instance
    

class BaykeUserUpdateAboutSerializer(BaykeUserUpdateAvatarSerializer):
    """ 修改个人简介 """
    class Meta:
        model = BaykeUser
        fields = ('about', )

    def update(self, instance, validated_data):
        instance.about = validated_data['about']
        instance.save()
        return instance
    

class SendEmailSerializer(serializers.Serializer):
    """ 发送邮件 """
    email = serializers.EmailField(max_length=80)
    
    def push_mail(self, validated_data):
        code = code_random()
        cache_code = cache.get_or_set(validated_data['email'], code, 300)
        push_main(cache_code, validated_data['email'])
        return True


class VerifyEmailSerializer(serializers.Serializer):
    """ 验证邮箱 """
    email = serializers.EmailField(max_length=80)
    code = serializers.CharField(
        max_length=bayke_settings.CODE_LENGTH,
        min_length=bayke_settings.CODE_LENGTH,
        write_only=True
    )

    def validate_email(self, email):
        user = get_user_model().objects.filter(email=email)
        if email == self.context['request'].user.email:
            raise serializers.ValidationError("邮箱地址似乎未改变哦？")
        if user.exists():
            raise serializers.ValidationError("该邮箱已被占用，请更换邮箱！")
        return email
    
    def validate(self, attrs):
        cache_code = cache.get(attrs['email'])
        if cache_code != attrs['code']:
            raise serializers.ValidationError("验证码有误！")
        return super().validate(attrs)
    

class BaykeUserBalancePushSerializer(serializers.Serializer):
    """ 余额充值序列化 """
    add_balance = serializers.DecimalField(max_digits=10, decimal_places=2)


'''
@file            :sites.py
@Description     :自定义AdminSite
@Date            :2023/09/02 21:45:32
@Author          :幸福关中 && 轻编程
@version         :v1.0
@EMAIL           :1158920674@qq.com
@WX              :baywanyun
'''

#!/usr/bin/env python
# -*- encoding: utf-8 -*-
'''
@文件    :sites.py
@说明    :自定义管理站点
@时间    :2023/10/31 10:03:58
@作者    :幸福关中&轻编程
@版本    :1.0
@微信    :baywanyun
'''


from django.contrib import admin
from django.utils.translation import gettext as _
from django.utils.translation import gettext_lazy
from baykeshop.conf import bayke_settings

from .forms import AdminLoginForm


class AdminSite(admin.AdminSite):
    """ 自定义AdminSite """
    site_header = gettext_lazy(bayke_settings.SITE_HEADER)
    site_title = gettext_lazy(bayke_settings.SITE_TITLE)
    index_title = gettext_lazy(bayke_settings.INDEX_TITLE)

    login_form = AdminLoginForm
    login_template = "system/login.html"
    index_template = "system/index.html"
    
    
    def get_app_list(self, request, app_label=None):
        if bayke_settings.CUSTOM_MENU:
            from baykeshop.common.menus import MenusMixins
            menu = MenusMixins()
            return menu.get_menus(request)
        return super().get_app_list(request)



from django.urls import path, include
from tinymce.views import TinyMCEImageUpload

urlpatterns = [
    path('', include('baykeshop.apps.shop.urls')),
    path('article/', include('baykeshop.apps.article.urls')),
    path('system/', include('baykeshop.apps.system.urls')),
    path('user/', include('baykeshop.apps.user.urls')),
    path('captcha/', include('captcha.urls')),
    path("tinymce/upload-image/", TinyMCEImageUpload.as_view(), name="tinymce-upload-image"),

    # 接口url
    path('api-auth/', include('rest_framework.urls'))
]